#include "FiveWin.ch"
#include "Ide.ch"

static aDbfBmps

//----------------------------------------------------------------------------//

function DbfNew( oMruDbfs )

   local oDlg, oGet, oType, oLen, oDec, oLbx, oBtnAdd, oBtnEdit, oCbx
   local cName    := Space( 10 )
   local cType    := "C"
   local nLen     := 10
   local nDec     :=  0
   local cField   := Space( 20 )
   local cTypes   := "CNLDM"
   local aLens    := { 10, 10, 1, 8, 8 }
   local cDbfName := Space( 12 )
   local lEditing := .f.
   local cDriver  := "DbfNtx"


   DEFINE DIALOG oDlg RESOURCE "DbfBuild"

   REDEFINE GET oGet VAR cName ID 110 OF oDlg

   REDEFINE COMBOBOX oType VAR cType  ITEMS { "C", "N", "L", "D", "M" } ;
      ON CHANGE ( nLen := aLens[ At( cType, cTypes ) ], oLen:Refresh() );
      ID 120 OF oDlg

   REDEFINE GET oLen VAR nLen PICTURE "9999" ID 130 OF oDlg

   REDEFINE GET oDec VAR nDec PICTURE "9"    ID 140 OF oDlg

   REDEFINE BUTTON oBtnAdd ID 150 OF oDlg ;
      ACTION AddField( oLbx, oGet, oBtnAdd, oBtnEdit,;
                       @cName, cType, nLen, nDec, @lEditing )

   REDEFINE BUTTON ID 160 OF oDlg ACTION oDlg:End()

   REDEFINE LISTBOX oLbx VAR cField ID 170 OF oDlg

   REDEFINE BUTTON oBtnEdit ID 180 OF oDlg ;
      ACTION EditField( oBtnAdd, oBtnEdit,;
                        cField, @cName, @cType, @nLen, @nDec, @lEditing,;
                        oGet, oType, oLen, oDec )

   REDEFINE BUTTON ID 190 OF oDlg ACTION oLbx:Del()

   REDEFINE GET cDbfName ID 210 OF oDlg

   REDEFINE COMBOBOX oCbx VAR cDriver ;
      ITEMS { "DbfNtx", "DbfCdx", "DbfMdx", "DbfNdx", "DbPx" } ;
      ID 230 OF oDlg

   REDEFINE BUTTON ID 220 OF oDlg ;
      ACTION ( BuildDbf( RTrim( cDbfName ), oLbx, cDriver ),;
               oMruDbfs:Save( cDbfName ) )

   ACTIVATE DIALOG oDlg CENTERED

return nil

//----------------------------------------------------------------------------//

static function AddField( oLbx, oGet, oBtnAdd, oBtnEdit,;
                          cName, cType, nLen, nDec, lEditing )

   if Empty( cName )
      MsgInfo( "I need a field name", "Sorry" )
   else
      if ! lEditing
         oLbx:Add( xPadR( cName, 100 ) + Chr( 9 ) + cType + ;
                   xPadL( Str( nLen, 3 ), 50 ) + xPadL( Str( nDec, 1 ), 20 ),;
                   oLbx:GetPos() )
			else
         oLbx:Modify( xPadR( cName, 100 ) + Chr( 9 ) + cType + ;
                      xPadL( Str( nLen, 3 ), 50 ) + xPadL( Str( nDec, 1 ), 20 ) )
         oBtnAdd:SetText( "&Add" )
         oBtnEdit:Enable()
         lEditing = .f.
      endif
      cName = Space( 10 )
      oGet:Refresh()
      oGet:SetFocus()
   endif

return nil

//----------------------------------------------------------------------------//

static function BuildDbf( cDbfName, oLbx, cDriver )

   local aFields := {}
   local n

   if Empty( cDbfName )
      MsgAlert( "I need a DBF name", "Sorry" )
      return nil
   endif

   if Len( oLbx:aItems ) == 0
      MsgAlert( "No fields defined", "Sorry" )
      return nil
   endif

   if At( ".", cDbfName ) == 0
      cDbfName += ".dbf"
   endif

   if File( cDbfName )
      if ! MsgYesNo( "Create it again?", "That DBF already exists" )
         return nil
      endif
   endif

   for n = 1 to Len( oLbx:aItems )
      AAdd( aFields, FieldInfo( AllTrim( oLbx:aItems[ n ] ) ) )
   next

   RddSetDefault( cDriver )
   DbCreate( cDbfName, aFields )
   RddSetDefault( "DbfNtx" )

   MsgInfo( "DBF created!", "AllRight" )

return nil

//----------------------------------------------------------------------------//

static function FieldInfo( cItem )

return { StrToken( cItem, 1 ),;
         Right( StrToken( cItem, 2 ), 1 ),;
         Val( StrToken( cItem, 3 ) ),;
         Val( StrToken( cItem, 4 ) ) }

//----------------------------------------------------------------------------//

static function EditField( oBtnAdd, oBtnEdit, cField,;
                           cName, cType, nLen, nDec, lEditing,;
                           oName, oType, oLen, oDec )

   if ! Empty( cField )
      oBtnAdd:SetText( "&Replace" )
      oBtnEdit:Disable()
      lEditing = .t.

      cName = StrToken( cField, 1 )
      cType = Right( StrToken( cField, 2 ), 1 )
      nLen  = Val( StrToken( cField, 3 ) )
      nDec  = Val( StrToken( cField, 4 ) )

      oName:Refresh()
      oType:Refresh()
      oLen:Refresh()
      oDec:Refresh()
   else
      MsgInfo( "Select a field to edit", "Please" )
   endif

return nil

//----------------------------------------------------------------------------//

function DbfOpen( cDbfName, cDriver, oMruDbfs )

   local oWndBrw, oBar, oBrw, oIco, oWndRec, oFont, oRec
   local nFilter
   local cAlias

   DEFAULT cDriver  := "DbfNtx"
   DEFAULT aDbfBmps := { LoadBitmap( GetResources(), "NoDeleted" ),;
                         LoadBitmap( GetResources(), "Deleted" ) }

   if Empty( cDbfName )
      cDbfName := cGetFile( "Standard dbf (*.dbf) | *.dbf |" + ;
                            "FoxPro cdx (*.dbf) | *.dbf |" + ;
                            "dBase IV mdx (*.dbf) | *.dbf |" + ;
                            "dBase III+ ndx (*.dbf) | *.dbf |" + ;
                            "Paradox db (*.db) | *.db", ;
                            "Select a DataBase to use", 1, "..\samples" )

      cDriver = DRIVERS( IDE_DRIVERS )[ nGetFileFilter() ]
   endif

   if ! Empty( cDbfName )

      USE ( cDbfName ) NEW VIA ( cDriver ) SHARED ;
         ALIAS ( cCheckArea( cNoExt( cNoPath( cDbfName ) ) ) )

      cAlias = Alias()

      DEFINE ICON oIco RESOURCE "DataBase"

      DEFINE WINDOW oWndBrw FROM 1, 1 TO 23, 33 ;
         TITLE "DataBase: " + cNoExt( cNoPath( cDbfName ) ) ;
         ICON oIco MDICHILD

      DEFINE BUTTONBAR oBar 3D SIZE 33, 33 OF oWndBrw

      DEFINE BUTTON RESOURCE "New" OF oBar ;
         ACTION ( oBrw:RecAdd(), oBrw:Refresh() ) ;
         MESSAGE "Add a new record"

      DEFINE BUTTON RESOURCE "Edit" OF oBar ;
         MESSAGE "Edit current record" ;
         ACTION  oWndRec := RecEdit( "Record: " + cAlias, oBrw )

      DEFINE BUTTON RESOURCE "Del" OF oBar ;
         MESSAGE "Delete current record" ;
         ACTION ( ( cAlias )->( DelToggle() ), oBrw:DrawSelect() )

      DEFINE BUTTON RESOURCE "Prg" GROUP OF oBar ;
         MESSAGE "Generate source code" ACTION DbfGen( oBrw:cAlias )

      DEFINE BUTTON RESOURCE "Top" GROUP OF oBar ;
         ACTION oBrw:GoTop() ;
         MESSAGE "Go to first record"

      DEFINE BUTTON RESOURCE "Rewind" OF oBar ;
         ACTION oBrw:PageUp() ;
         MESSAGE "Go to previous page"

      DEFINE BUTTON RESOURCE "Prev" OF oBar ;
         ACTION oBrw:GoUp() ;
         MESSAGE "Go previous record"

      DEFINE BUTTON RESOURCE "Next" OF oBar ;
         ACTION oBrw:GoDown() ;
         MESSAGE "Go to next record"

      DEFINE BUTTON RESOURCE "Forward" OF oBar ;
         ACTION oBrw:PageDown() ;
         MESSAGE "Go to next page"

      DEFINE BUTTON RESOURCE "Bottom" OF oBar ;
         ACTION oBrw:GoBottom() ;
         MESSAGE "Go to last record"

      DEFINE BUTTON RESOURCE "Font" GROUP OF oBar ;
         ACTION oBrw:SelFont() ;
         MESSAGE "Select a new font"

      DEFINE BUTTON RESOURCE "Clr" OF oBar ;
         ACTION ( oBrw:SelColor(),;
                  If( oWndRec != nil,;
                  oWndRec:oControl:SetColor( oBrw:nClrText, oBrw:nClrPane ),) ) ;
         MESSAGE "Select a new backcolor"

      SET MESSAGE OF oWndBrw TO "Browsing DataBase" 

      DEFINE MSGITEM oRec OF oWndBrw:oMsgBar ;
         PROMPT "Rec: 1/" + LTrim( ( cAlias )->( Str( RecCount() ) ) ) ;
         SIZE 90

      DEFINE MSGITEM OF oWndBrw:oMsgBar ;
         PROMPT ( cAlias )->( RddName() ) SIZE 60

      DEFINE FONT oFont NAME "Arial" SIZE 6, 15 BOLD

      @ 0, 0 LISTBOX oBrw FIELDS OF oWndBrw ;
         ON CHANGE ( If( GetWndAreas() != nil, GetWndAreas():oControl:Refresh(),),;
                     If( oWndRec != nil, oWndRec:oControl:Refresh(),),;
                     oRec:SetText( "Rec: " + LTrim( ( cAlias )->( Str( RecNo() ) ) ) ;
                                   + "/" + LTrim( ( cAlias )->( Str( RecCount() ) ) ) ) ) ;
         ON DBLCLICK oWndRec := RecEdit( "Record: " + cAlias, oBrw ) ;
         SIZE 400, 400 FONT oFont

      oBrw:bLine = { | aTemp | aTemp := _aFields( oBrw ),;
                               AAdd( aTemp, nil ), AIns( aTemp, 1 ),;
                               aTemp[ 1 ] := If( ( cAlias )->( Deleted() ),;
                               aDbfBmps[ 2 ], aDbfBmps[ 1 ] ), aTemp }

      AAdd( oBrw:aJustify, .f. )
      AIns( oBrw:aJustify, 1 )
      oBrw:aJustify[ 1 ] = .f.

      AAdd( oBrw:aColSizes, nil )
      AIns( oBrw:aColSizes, 1 )
      oBrw:aColSizes[ 1 ] = 16

      AAdd( oBrw:aHeaders, nil )
      AIns( oBrw:aHeaders, 1 )
      oBrw:aHeaders[ 1 ] = ""

      oWndBrw:SetControl( oBrw )

      ACTIVATE WINDOW oWndBrw ;
         VALID ( If( oWndRec != nil, oWndRec:End(),),;
                 oMruDbfs:Save( cDBfName ), oBrw:lCloseArea() )

      if GetWndAreas() != nil
         GetWndAreas():oControl:Refresh()  // We updates the info on the WorkAreas
      endif                            // Inspector

   endif

return nil

//----------------------------------------------------------------------------//

static function DelToggle()

   if RLock()
      if Deleted()
         DbReCall()
      else
         DbDelete()
      endif
      UNLOCK
   endif

return nil

//----------------------------------------------------------------------------//
// Checks for a possible area name conflict

static function cCheckArea( cDbfName )

   local n      := 2
   local cAlias := cDbfName

   while Select( cAlias ) != 0
      cAlias = cDbfName + AllTrim( Str( n++ ) )
   end

return cAlias

//----------------------------------------------------------------------------//
// Remember to use 'exit' procedures to asure that resources are
// freed on a possible application error

exit procedure FreeResources()

   if aDbfBmps != nil
      AEval( aDbfBmps, { | hBmp | DeleteObject( hBmp ) } )
   endif

return 

//----------------------------------------------------------------------------//

static function DbfGen( cAlias )

   local cPrg  := "// FiveWin: Automatic code generation " + CRLF + ;
                  "// for " + cAlias + ".dbf" + CRLF + CRLF + ;
                  '#include "FiveWin.ch"' + CRLF + CRLF + ;
                  "//-----------------------------------------" + ;
                  "----------------------------------//" + CRLF + CRLF + ;
                  "function Main()" + CRLF + CRLF + ;
                  "   SET 3DLOOK ON" + CRLF + CRLF + ;
                  '   if ! File( "' + cAlias + '.dbf" )' + CRLF + ;
                  '      DbCreate( "' + cAlias + '",;' + CRLF

   local aInfo := ( cAlias )->( DbStruct() )
   local n, cTempName

   for n = 1 to Len( aInfo )
      cPrg += Space( 20 ) + If( n == 1, "{ ", "  " ) + '{ "' + ;
              aInfo[ n ][ 1 ] + '", "' + ;
              aInfo[ n ][ 2 ] + '", ' + ;
              AllTrim( Str( aInfo[ n ][ 3 ] ) ) + ", " + ;
              AllTrim( Str( aInfo[ n ][ 4 ] ) ) + "}" + ;
              If( n < Len( aInfo ), ",;", " } )" ) + CRLF
   next

   cPrg += "   endif" + CRLF + CRLF + ;
           "   USE " + cAlias + CRLF + ;
           '   Browse( "DataBase: ' + cAlias + ;
           '",, { ||' + cAlias + '->( DbAppend() ) } )' + CRLF + ;
           "   USE" + CRLF + CRLF + ;
           "return nil" + CRLF + CRLF + ;
           "//-----------------------------------------" + ;
           "----------------------------------//"

   MemoWrit( "Programs\" + ;
             ( cTempName := cGetNewFileName( SubStr( cAlias, 1, 6 ), "prg" ) ),;
             cPrg )

   PrgOpen( "Programs\" + cTempName, GetMruPrgs() )

return nil

//----------------------------------------------------------------------------//
