#include "FiveWin.ch"

#define BAR_HEIGHT     28
#define GRAY_BRUSH      2

#define BAR_TOP         1
#define BAR_LEFT        2
#define BAR_RIGHT       3
#define BAR_DOWN        4
#define BAR_FLOAT       5

#define SM_CYBORDER     6
#define SM_CYCAPTION    4

//----------------------------------------------------------------------------//

CLASS TBar FROM TControl

   DATA   nGroups, nMode
   DATA   nBtnWidth, nBtnHeight
   DATA   l3D

   CLASSDATA lRegistered AS LOGICAL

   METHOD New( oWnd, nBtnWidth, nBtnHeight, l3D, cMode, oCursor ) CONSTRUCTOR

   METHOD NewAt( nRow, nCol, nWidth, nHeight, nBtnWidth, nBtnHeight, oWnd,;
                 l3D, cMode, oCursor ) CONSTRUCTOR

   METHOD Add( oBtnBmp ) INLINE  AAdd( ::aControls, oBtnBmp )

   METHOD Adjust()

   METHOD BtnAdjust()

   METHOD Display() INLINE ::BeginPaint(), ::Paint(), ::EndPaint(), 0

   METHOD cGenPRG()
   METHOD GetBtnTop( lNewGroup )
   METHOD GetBtnLeft( lNewGroup )

   METHOD Float()
   METHOD GoDown()
   METHOD GoLeft()
   METHOD GoRight()
   METHOD GoTop()

   METHOD RButtonDown( nRow, nCol, nFlags )

   METHOD MouseMove( nRow, nCol, nKeyFlags ) INLINE ;
                     Super:MouseMove( nRow, nCol, nKeyFlags ),;
                     ::oWnd:SetMsg(), 0

   METHOD Paint()

   METHOD FloatClick( nRow, nCol, oWnd )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( oWnd, nBtnWidth, nBtnHeight, l3D, cMode, oCursor ) CLASS TBar

   local oRect := oWnd:GetCliRect()

   DEFAULT nBtnWidth := BAR_HEIGHT, nBtnHeight := BAR_HEIGHT,;
           l3D := .f., cMode := "TOP"

   ::nStyle      = nOR( WS_BORDER, WS_CHILD, WS_VISIBLE )
   ::aControls   = {}
   ::nGroups     = 0
   ::oWnd        = oWnd
   ::nTop        = If( cMode == "BOTTOM", oRect:nBottom - nBtnHeight, -1 )
   ::nLeft       = If( cMode == "RIGHT", oRect:nRight - nBtnWidth - ;
                   If( l3D, 3, 0 ), -1 )
   ::nBottom     = If( cMode == "TOP", nBtnHeight, oRect:nBottom + 1 )
   ::nRight      = If( cMode == "TOP" .or. cMode == "BOTTOM",;
                       oRect:nRight,;
                   If( cMode == "LEFT", nBtnWidth + If( l3D, 3, 0 ), oRect:nRight + 1 ) )
   ::nBtnWidth   = nBtnWidth
   ::nBtnHeight  = nBtnHeight
   ::nId         = ::GetNewId()
   ::lDrag       = .f.
   ::lCaptured   = .f.
   ::nClrPane    = If( l3D, CLR_LIGHTGRAY, CLR_GRAY )
   ::lVisible    = .t.
   ::l3D         = l3D
   ::nMode       = AScan( { "TOP", "LEFT", "RIGHT", "BOTTOM", "FLOAT" }, cMode )
   ::oCursor     = oCursor
   ::lValidating = .f.

   ::SetColor( ::nClrText, ::nClrPane )

   ::Register( nOR( CS_VREDRAW, CS_HREDRAW ) )
   ::Create()

   if oWnd != nil
      oWnd:oBar = Self
   endif

return nil

//----------------------------------------------------------------------------//

METHOD NewAt( nRow, nCol, nWidth, nHeight, nBtnWidth, nBtnHeight, oWnd, l3D,;
              cMode, oCursor ) CLASS TBar

   local oRect := oWnd:GetCliRect()

   DEFAULT nBtnWidth := BAR_HEIGHT, nBtnHeight := BAR_HEIGHT,;
           nHeight := BAR_HEIGHT,;
           l3D := .f., cMode := "TOP"

   ::nStyle      = nOR( WS_BORDER, WS_CHILD, WS_VISIBLE )
   ::aControls   = {}
   ::nGroups     = 0
   ::oWnd        = oWnd
   ::nTop        = nRow
   ::nLeft       = nCol
   ::nBottom     = nRow + nHeight - 1
   ::nRight      = nCol + nWidth - 1
   ::nBtnWidth   = nBtnWidth
   ::nBtnHeight  = nBtnHeight
   ::nId         = ::GetNewId()
   ::lDrag       = .f.
   ::lCaptured   = .f.
   ::nClrPane    = If( l3D, CLR_LIGHTGRAY, CLR_GRAY )
   ::lVisible    = .t.
   ::l3D         = l3D
   ::nMode       = AScan( { "TOP", "LEFT", "RIGHT", "BOTTOM", "FLOAT" }, cMode )
   ::oCursor     = oCursor
   ::lValidating = .f.

   ::SetColor( ::nClrText, ::nClrPane )

   ::Register( nOR( CS_VREDRAW, CS_HREDRAW ) )
   ::Create()

return nil

//----------------------------------------------------------------------------//

METHOD BtnAdjust() CLASS TBar

   local n, nGroups := 0
   local nRow := If( ::l3D, 2, -1 )
   local nCol := If( ::l3D, 2, -1 )

   do case
      case ::nMode == BAR_TOP .or. ::nMode == BAR_DOWN
           for n = 1 to Len( ::aControls )
              nGroups += If( ::aControls[ n ]:lGroup, 1, 0 )
              ::aControls[ n ]:Move( If( ::l3D, 3, 0 ) - 1,;
                                     nCol + ( nGroups * 15 ) )
              ::aControls[ n ]:Refresh()
              nCol    += ::nBtnWidth
           next

      case ::nMode == BAR_LEFT .or. ::nMode == BAR_RIGHT
           for n = 1 to Len( ::aControls )
              nGroups += If( ::aControls[ n ]:lGroup, 1, 0 )
              ::aControls[ n ]:Move( nRow + ( nGroups * 15 ),;
                                     If( ::l3D, 3, 0 ) - 1 )
              ::aControls[ n ]:Refresh()
              nRow += ::nBtnHeight - If( ::l3D, 3, -1 )
           next

      case ::nMode == BAR_FLOAT
           nRow = 0
           nCol = 0
           for n = 1 to Len( ::aControls )
              ::aControls[ n ]:Move( nRow, nCol )
              nRow = If( nCol == ::nBtnWidth - If( ::l3D, 2, 0 ), nRow + ;
                         ::nBtnHeight - If( ::l3D, 5, -1 ), nRow )
              nCol = If( nCol == 0, ::nBtnWidth - If( ::l3D, 2, 0 ), 0 )
           next

   endcase

return nil

//----------------------------------------------------------------------------//

METHOD cGenPRG() CLASS TBar

   local cPrg := ""
   local n

   cPrg += CRLF + CRLF + "   DEFINE BUTTONBAR oBar OF oWnd " + ;
           If( ::nMode != BAR_TOP,;
               { "TOP", "LEFT", "RIGHT", "DOWN", "FLOAT" }[ ::nMode ],)

   AEval( ::aControls, { | oCtrl | cPrg += oCtrl:cGenPRG() } )

return cPrg

//----------------------------------------------------------------------------//

METHOD GetBtnLeft( lNewGroup ) CLASS TBar

   local nPos := 0

   do case
      case ::nMode == BAR_TOP .or. ::nMode == BAR_DOWN
           nPos = ( Len( ::aControls ) * ::nBtnWidth ) + ;
                  ( If( lNewGroup, ++::nGroups, ::nGroups ) * 15 ) + ;
                  If( ::l3D, 2, 0 )

      case ::nMode == BAR_LEFT .or. ::nMode == BAR_RIGHT
           nPos = If( ::l3D, 2, 0 )

      case ::nMode == BAR_FLOAT
           nPos = If( Len( ::aControls ) > 0,;
                  If( ATail( ::aControls ):nRight + ::nBtnWidth > ::nRight,;
                      -1, ( Len( ::aControls ) * ::nBtnWidth ) - 1 ), -1 )
   endcase

return nPos

//----------------------------------------------------------------------------//

METHOD GetBtnTop( lNewGroup ) CLASS TBar

   local nPos     := 0
   local nButtons := Len( ::aControls )

   do case
      case ::nMode == BAR_TOP .or. ::nMode == BAR_DOWN
           nPos = If( ::l3D, 2, -1 )

      case ::nMode == BAR_LEFT .or. ::nMode == BAR_RIGHT
           nPos = If( ::l3D, 2, 0 ) + ;
                  ( nButtons * ( ::nBtnHeight - If( ::l3D, 3, 0 ) ) ) + ;
                  ( If( lNewGroup, ++::nGroups, ::nGroups ) * 15 )
   endcase

return nPos

//----------------------------------------------------------------------------//

METHOD RButtonDown( nRow, nCol, nFlags ) CLASS TBar

   local oMenu
   local aCoors := { nRow, nCol }

   if ::bRClicked != nil
      return Eval( ::bRClicked, nRow, nCol, nFlags )
   endif

   ClientToScreen( ::hWnd, aCoors )

   MENU oMenu POPUP
      if ::nMode != BAR_TOP
         MENUITEM "&Top"    ACTION ::GoTop()
      endif
      if ::nMode != BAR_LEFT
         MENUITEM "&Left"   ACTION ::GoLeft()
      endif
      if ::nMode != BAR_RIGHT
          MENUITEM "&Right"  ACTION ::GoRight()
      endif
      if ::nMode != BAR_DOWN
         MENUITEM "&Bottom" ACTION ::GoDown()
      endif
      SEPARATOR
      MENUITEM "&Float"  ACTION ::Float()
   ENDMENU

   ACTIVATE POPUP oMenu AT aCoors[ 1 ], aCoors[ 2 ] OF Self

return nil

//----------------------------------------------------------------------------//

METHOD GoDown() CLASS TBar

   local oRect

   if ::nMode != BAR_DOWN
      ::nMode = BAR_DOWN
      oRect = ::oWnd:GetCliRect()
      oRect:nBottom -= If( ::oWnd:oMsgBar != nil, ::oWnd:oMsgBar:Height(), 0 )
      oRect:nTop = oRect:nBottom - ::nBtnHeight - 1
      oRect:nLeft--

      ::SetCoors( oRect )
      ::BtnAdjust()

      if ::oWnd:ChildLevel( TMdiFrame() ) != 0 .and. ::oWnd:oWndClient != nil
         ::oWnd:oWndClient:Refresh()
      endif
      ::oWnd:Resize()

   endif

return nil

//----------------------------------------------------------------------------//

METHOD GoLeft() CLASS TBar

   local oRect

   if ::nMode != BAR_LEFT
      ::nMode = BAR_LEFT
      oRect = ::oWnd:GetCliRect()
      oRect:nTop--
      oRect:nLeft--
      oRect:nBottom -= If( ::oWnd:oMsgBar != nil, ::oWnd:oMsgBar:Height(), 0 )
      oRect:nRight = ::nBtnWidth - 1 + If( ::l3D, 4, 0 )

      ::oWnd:Resize()
      if ::oWnd:ChildLevel( TMdiFrame() ) != 0 .and. ::oWnd:oWndClient != nil
         ::oWnd:oWndClient:Refresh()
      endif

      ::SetCoors( oRect )
      ::BtnAdjust()
      ::Refresh()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GoRight() CLASS TBar

   local oRect

   if ::nMode != BAR_RIGHT
      ::nMode = BAR_RIGHT
      oRect = ::oWnd:GetCliRect()
      oRect:nTop--
      oRect:nLeft = oRect:nRight - ::nBtnWidth - If( ::l3D, 3, 0 ) - 1
      oRect:nBottom -= If( ::oWnd:oMsgBar != nil, ::oWnd:oMsgBar:Height(), 0 )

      ::oWnd:Resize()
      if ::oWnd:ChildLevel( TMdiFrame() ) != 0 .and. ::oWnd:oWndClient != nil
         ::oWnd:oWndClient:Refresh()
      endif

      ::SetCoors( oRect )
      ::BtnAdjust()
      ::Refresh()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GoTop() CLASS TBar

   local oRect

   if ::nMode != BAR_TOP
      ::nMode = BAR_TOP
      oRect = ::oWnd:GetCliRect()
      oRect:nTop--
      oRect:nLeft--
      oRect:nBottom = ::nBtnHeight

      ::oWnd:Resize()
      if ::oWnd:ChildLevel( TMdiFrame() ) != 0 .and. ::oWnd:oWndClient != nil
         ::oWnd:oWndClient:Refresh()
      endif

      ::SetCoors( oRect )
      ::BtnAdjust()
      ::Refresh()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Float() CLASS TBar

   local oWnd
   local oBar := Self
   local n

   local nWidth
   local nHeight
   local nAdjust   := If( ::l3D, - 5, 1 )
   local nControls := If( Len( ::aControls ) % 2 == 0, ;
                          ( Len( ::aControls ) / 2 ), ;
                          ( ( Len( ::aControls ) + 1 ) / 2 ) )

   if ::nMode != BAR_FLOAT
      ::nMode = BAR_FLOAT

      DEFINE WINDOW oWnd ;
         TITLE "" OF ::oWnd ;
         BORDER NONE ;
         NOZOOM NOICONIZE

      REDEFINE SYSMENU OF oWnd
         SEPARATOR
         MENUITEM "&Top"    ACTION ( AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), oWnd:End(), ::Show(), ::GoTop() )
         MENUITEM "&Bottom" ACTION ( AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), oWnd:End(), ::Show(), ::GoDown() )
         MENUITEM "&Left"   ACTION ( AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), oWnd:End(), ::Show(), ::GoLeft() )
         MENUITEM "&Right"  ACTION ( AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), oWnd:End(), ::Show(), ::GoRight() )
      ENDSYSMENU

      for n = 1 to Len( ::aControls )
         SetParent( ::aControls[ n ]:hWnd, oWnd:hWnd )
      next

      nWidth  = ( oBar:nBtnWidth * 2 ) - If( ::l3D, 2, - 2 )
      nHeight = GetSysMetrics( SM_CYCAPTION ) + 1 + ;
                ( ( oBar:nBtnHeight + nAdjust ) * nControls ) + 10

      oWnd:Move( 90, 30, nWidth, nHeight, .t. )

      oWnd:bRClicked = { | nRow, nCol | ::FloatClick( nRow, nCol, oWnd ) }

      ::Hide()
      ::oWnd:Resize()

      oWnd:Show()
      ::BtnAdjust()
      oWnd:SetFocus()

      if ::ChildLevel( TMdiFrame() ) != 0 .and. ::oWnd:oWndClient != nil
         ::oWnd:oWndClient:Refresh()
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Paint() CLASS TBar

   if ! ::l3D
      Super:Paint()
   else
      WndRaised( ::hWnd, ::hDC )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD FloatClick( nRow, nCol, oWnd ) CLASS TBar

   local oMenu
   local aCoors := { nRow, nCol }
   local lEnd   := .f.

   ClientToScreen( oWnd:hWnd, aCoors )

   MENU oMenu POPUP
      MENUITEM "&Top"    ACTION ( oWnd:Hide(), AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), lEnd := .t., ::Show(), ::GoTop() )
      MENUITEM "&Left"   ACTION ( oWnd:Hide(), AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), lEnd := .t., ::Show(), ::GoLeft() )
      MENUITEM "&Right"  ACTION ( oWnd:Hide(), AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), lEnd := .t., ::Show(), ::GoRight() )
      MENUITEM "&Bottom" ACTION ( oWnd:Hide(), AEval( ::aControls, { | oCtrl | SetParent( oCtrl:hWnd, ::hWnd ) } ), lEnd := .t., ::Show(), ::GoDown() )
   ENDMENU

   ACTIVATE POPUP oMenu AT aCoors[ 1 ], aCoors[ 2 ] OF oWnd

   if lEnd
      oWnd:End()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Adjust() CLASS TBar

   local oMsgBar := ::oWnd:oMsgBar

   BarAdjust( ::hWnd, ::nMode, oMsgBar != nil,;
              If( oMsgBar != nil, oMsgBar:Height(),) )

return nil

//----------------------------------------------------------------------------//
