#include "FiveWin.ch"

#define LTGRAY_BRUSH       1
#define GRAY_BRUSH         2

#define WM_ERASEBKGND     20
#define WM_LBUTTONDBLCLK 515  // 0x203
#define WM_CTLCOLOR       25  // 0x19       // Don't remove Color Control
#define WM_MENUSELECT    287

#define GW_HWNDNEXT        2
#define GW_CHILD           5

#define SC_CLOSE       61536   // 0xF060
#define SC_NEXT        61504
#define SC_KEYMENU     61696   // 0xF100

#define WM_SYSCOMMAND    274   // 0x0112

#define DM_GETDEFID  WM_USER

static nId := 100
static nPoint
static nMRow, nMCol
static nXGrid := 8, nYGrid := 8

static lDragging := .f.

//----------------------------------------------------------------------------//

CLASS TControl FROM TWindow

   DATA   bSetGet, bChange, bDragBegin
   DATA   cCaption
   DATA   lCaptured, lDrag, lMouseDown, lUpdate
   DATA   nLastRow, nLastCol

   METHOD Change() VIRTUAL

   METHOD Click() INLINE  ::oWnd:AEvalWhen()

   METHOD cToChar( cCtrlClass )
   METHOD Init( hDlg )

   METHOD Default()

   METHOD EraseBkGnd( hDC )

   METHOD FWLostFocus()

   METHOD GetDlgCode( nLastKey )

   METHOD GetNewId() INLINE ++nId

   METHOD GotFocus( hCtlLost )

   MESSAGE HelpTopic METHOD __HelpTopic()

   METHOD LostFocus() INLINE ::SetMsg(), Super:LostFocus()

   METHOD End()

   METHOD HandleEvent( nMsg, nWParam, nLParam ) EXTERN ;
                                    CtlHandleEvent( nMsg, nWParam, nLParam )

   METHOD KeyChar( nKey, nFlags )
   METHOD KeyDown( nKey, nFlags )

   MESSAGE SetFocus METHOD _SetFocus()

   METHOD Colors( hDC )

   METHOD DragBegin( nRow, nCol, nKeyFlags )

   METHOD DrawItem( nPStruct ) VIRTUAL
   METHOD FillMeasure()        VIRTUAL

   METHOD KillFocus( hCtlFocus )

   METHOD Set3DLook() INLINE Ctl3DLook( ::hWnd )

   METHOD VarPut( uVal ) INLINE  If( ValType( ::bSetGet ) == "B",;
                                 Eval( ::bSetGet, uVal ),)

   METHOD VarGet() INLINE If( ValType( ::bSetGet ) == "B", Eval( ::bSetGet ),)

   METHOD LButtonDown( nRow, nCol, nKeyFlags )

   METHOD LButtonUp( nRow, nCol, nKeyFlags )

   METHOD MouseMove( nRow, nCol, nKeyFlags )

   METHOD ForWhen()

   METHOD Display() VIRTUAL

   METHOD Paint() VIRTUAL

   METHOD SysCommand( nType, nLoWord, nHiWord )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD cToChar( cCtrlClass ) CLASS TControl

   local n := GetDlgBaseUnits()

   DEFAULT cCtrlClass := ::ClassName(),;
           ::cCaption := "",;
           ::nId      := ::GetNewId(),;
           ::nStyle   := nOR( WS_CHILD, WS_VISIBLE, WS_TABSTOP )

return cCtrl2Chr( Int( 2 * 8 * ::nTop    / nHiWord( n ) ),;
                  Int( 2 * 4 * ::nLeft   / nLoWord( n ) ),;
                  Int( 2 * 8 * ::nBottom / nHiWord( n ) ),;
                  Int( 2 * 4 * ::nRight  / nLoWord( n ) ),;
                  ::nId, ::nStyle, cCtrlClass, ::cCaption )

//----------------------------------------------------------------------------//

METHOD Init( hDlg ) CLASS TControl

   local oRect

   DEFAULT ::lActive := .t., ::lDrag := .f., ::lCaptured := .f.

   if( ( ::hWnd := GetDlgItem( hDlg, ::nId ) ) != 0 )
      oRect     = ::GetRect()
      ::nTop    = oRect:nTop
      ::nLeft   = oRect:nLeft
      ::nBottom = oRect:nBottom
      ::nRight  = oRect:nRight

      If( ::lActive, ::Enable(), ::Disable() )

      ::Link()

      if ::oFont != nil
         ::SetFont( ::oFont )
      else
         ::GetFont()
      endif

   else
     #define NOVALID_CONTROLID   1
     Eval( ErrorBlock(), _FWGenError( NOVALID_CONTROLID, "No: " + ;
                                      Str( ::nId, 6 ) ) )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD _SetFocus() CLASS TControl

   local hCtrlNext

   if ::lWhen()
      SetFocus( ::hWnd )
   else
      hCtrlNext = GetWindow( ::hWnd, GW_HWNDNEXT )
      if GetParent( hCtrlNext ) != ::oWnd:hWnd
         hCtrlNext = GetWindow( ::oWnd:hWnd, GW_CHILD )
      endif
      SetFocus( hCtrlNext )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Colors( hDC ) CLASS TControl

   DEFAULT ::nClrText := GetTextColor( hDC ),;
           ::nClrPane := GetBkColor( hDC ),;
           ::oBrush   := TBrush():New( ,::nClrPane,)

   SetTextColor( hDC, ::nClrText )
   SetBkColor( hDC,   ::nClrPane )

return ::oBrush:hBrush

//----------------------------------------------------------------------------//

METHOD Default() CLASS TControl

   ::lDrag     = .f.
   ::lCaptured = .f.

return nil

//----------------------------------------------------------------------------//

METHOD LButtonDown( nRow, nCol, nKeyFlags ) CLASS TControl

   ::lMouseDown = .t.
   ::nLastRow   = nRow
   ::nLastCol   = nCol

   if ::lDrag
      if ! ::lCaptured
         SetFocus( ::hWnd )
         ::Capture()
         ::lCaptured = .t.
         nMRow  = nRow
         nMCol  = nCol
         nPoint = 0
         CtrlDrawFocus( ::hWnd )
      endif
      return 0
   else
      return Super:LButtonDown( nRow, nCol, nKeyFlags )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD LButtonUp( nRow, nCol, nKeyFlags ) CLASS TControl

   local aPoint := { nRow, nCol }

   ::lMouseDown = .f.

   if lDragging
      lDragging = .f.
      ReleaseCapture()

      ClientToScreen( ::hWnd, aPoint )
      SendMessage( WindowFromPoint( aPoint[ 2 ], aPoint[ 1 ] ),;
                   FM_DROPOVER, nKeyFlags, nMakeLong( nRow, nCol ) )

      return nil
   endif

   if ::lDrag
      if ::lCaptured
         ReleaseCapture()
         ::lCaptured = .f.
         if nPoint != 0
            CtrlDrawFocus( ::hWnd, nLoWord( nPoint ) - nMRow,;
                                   nHiWord( nPoint ) - nMCol )
            ::Move( ::nTop + nRow - nMRow, ::nLeft + nCol - nMCol,,, .t. )
            nPoint = nil
         else
            CtrlDrawFocus( ::hWnd, 0, 0 )
         endif
      endif
      return 0
   endif

return nil

//----------------------------------------------------------------------------//

METHOD MouseMove( nRow, nCol, nKeyFlags ) CLASS TControl

   local lXInc, lYInc

   DEFAULT ::lMouseDown := .f.

   if ::lDrag
      if ::lCaptured
         CursorCatch()
         lXInc = Abs( nLoWord( nPoint ) - nRow ) > nXGrid
         lYInc = Abs( nHiWord( nPoint ) - nCol ) > nYGrid
         if lXInc .or. lYInc
            if nPoint == 0
               CtrlDrawFocus( ::hWnd )
            else
               CtrlDrawFocus( ::hWnd, nLoWord( nPoint ) - nMRow,;
                                      nHiWord( nPoint ) - nMCol )
            endif
            nPoint = nMakeLong( nRow - ( nRow % nXGrid ),;
                                nCol - ( nCol % nYGrid ) )
            CtrlDrawFocus( ::hWnd, nLoWord( nPoint ) - nMRow,;
                                   nHiWord( nPoint ) - nMCol )
         endif
      endif
      return 0
   else
      if ::lMouseDown .and. ;
         ( Abs( nRow - ::nLastRow ) > 5 .or. Abs( nCol - ::nLastCol ) > 5 ) ;
         .and. ! Empty( ::oDragCursor )
         SetCursor( ::oDragCursor:hCursor )
         if ! lDragging
            ::DragBegin( nRow, nCol, nKeyFlags )
         endif
      else
         return Super:MouseMove( nRow, nCol, nKeyFlags )
      endif
   endif

return 0

//----------------------------------------------------------------------------//

METHOD End() CLASS TControl

   local nAt := If( ! Empty( ::aControls ),;
                AScan( ::oWnd:aControls, { | oCtrl | oCtrl:hWnd == Self:hWnd } ),;
                0 )

   if nAt != 0
      ADel( ::oWnd:aControls, nAt )
      ASize( ::oWnd:aControls, Len( ::oWnd:aControls ) - 1 )
   endif

return Super:End()

//----------------------------------------------------------------------------//

METHOD KeyChar( nKey, nFlags ) CLASS TControl

   local nDlgCode
   local nDefButton

   do case
      case nKey == VK_TAB .and. GetKeyState( VK_SHIFT )
           ::oWnd:GoPrevCtrl( ::hWnd )
           return 0    // We don't want API default behavior

      case nKey == VK_TAB
           ::oWnd:GoNextCtrl( ::hWnd )
           return 0    // We don't want API default behavior

      case nKey == VK_ESCAPE
           ::oWnd:End()
           return 0

      case nKey == VK_RETURN
           if ( nDefButton := nLoWord( SendMessage( ::oWnd:hWnd, DM_GETDEFID ) ) ) != 0 .and. ;
                nDefButton != ::nId
              SendMessage( GetDlgItem( ::oWnd:hWnd, nDefButton ), WM_KEYDOWN, VK_RETURN )
              return 0
           endif
           return nil

      otherwise
           return Super:KeyChar( nKey, nFlags )
   endcase

return nil             // We want API default behavior

//----------------------------------------------------------------------------//

METHOD ForWhen() CLASS TControl

   ::oWnd:AEvalWhen()

   ::lCaptured = .f.

   // keyboard navigation
   if ::oWnd:nLastKey == VK_UP .or. ::oWnd:nLastKey == VK_DOWN ;
      .or. ::oWnd:nLastKey == VK_RETURN .or. ::oWnd:nLastKey == VK_TAB
      if GetKeyState( VK_SHIFT )
         ::oWnd:GoPrevCtrl( ::hWnd )
      else
         ::oWnd:GoNextCtrl( ::hWnd )
      endif
   else
      if Empty( GetFocus() )
         SetFocus( ::hWnd )
      endif
   endif

   ::oWnd:nLastKey = 0

return nil

//----------------------------------------------------------------------------//

METHOD KeyDown( nKey, nFlags ) CLASS TControl

   if ::lDrag
      do case
         case nKey == VK_DELETE
              if MsgYesNo( "Delete this control ?" )
                 ::End()
                 SysRefresh()
              endif
      endcase
   else
      return Super:KeyDown( nKey, nFlags )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD KillFocus( hCtlFocus ) CLASS TControl

   if ( hCtlFocus != 0 .and. ;
      GetParent( hCtlFocus ) == GetParent( ::hWnd ) .or. ;
      ( ::oWnd:oBar != nil .and. ;
      ::oWnd:oBar:hWnd == GetParent( hCtlFocus ) ) ) .and. ;
      ! ::oWnd:lValidating
         PostMessage( ::hWnd, FM_LOSTFOCUS )
   endif

return ::LostFocus()

//----------------------------------------------------------------------------//

METHOD EraseBkGnd( hDC ) CLASS TControl

   if ::oBrush != nil
      FillRect( hDC, GetClientRect( ::hWnd ), ::oBrush:hBrush )
      return 1
   endif

return nil

//----------------------------------------------------------------------------//

METHOD FWLostFocus() CLASS TControl

   if ! ::oWnd:lValidating
        ::oWnd:lValidating = .t.
        if ! ::lValid()
           SetFocus( ::hWnd )
        else
           ::ForWhen()
        endif
        ::oWnd:lValidating = .f.
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GetDlgCode( nLastKey ) CLASS TControl

   if .not. ::oWnd:lValidating
      if nLastKey == VK_RETURN .or. nLastKey == VK_TAB
         ::oWnd:nLastKey = nLastKey

      // don't do a else here with :nLastKey = 0
      // or WHEN does not work properly, as we pass here twice before
      // evaluating the WHEN
      endif
   endif

   if ::oWnd:oWnd == nil   // There is no folders !!!
      return nil           // standard GetDlgCode behavior
   else
      if ::oWnd:oWnd:ClassName() != "TFOLDER"
         return nil        // standard GetDlgCode behavior
      endif
   endif

return DLGC_WANTALLKEYS // It is the only way to have 100% control using Folders

//----------------------------------------------------------------------------//

METHOD GotFocus( hCtlLost )

   ::lFocused = .t.
   ::oWnd:nResult = Self
   ::SetMsg( ::cMsg )

   if ::bGotFocus != nil
      return Eval( ::bGotFocus )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD SysCommand( nType, nLoWord, nHiWord ) CLASS TControl

   local oFolder

   do case
      case nType == SC_KEYMENU      // Alt+... control accelerator pressed
           if ! Empty( oFolder := ::oWnd:oWnd ) .and. ;
              oFolder:ClassName() == "TFOLDER"
              oFolder:SetOption( oFolder:GetHotPos( nLoWord ) )
              return 0
           endif
   endcase

return nil

//----------------------------------------------------------------------------//

METHOD __HelpTopic() CLASS TControl

   if Empty( ::nHelpId )
      if ::oWnd != nil
         ::oWnd:HelpTopic()
      else
         HelpIndex()
      endif
   else
      HelpPopup( ::nHelpId )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD DragBegin( nRow, nCol, nKeyFlags ) CLASS TControl

   if ! lDragging
      lDragging = .t.
      SetCapture( ::hWnd )
   endif

   if ::bDragBegin != nil
      Eval( ::bDragBegin, nRow, nCol, nKeyFlags )
   endif

return nil

//----------------------------------------------------------------------------//
