#include "FiveWin.ch"
#include "Sql.ch"

//----------------------------------------------------------------------------//

CLASS TDbOdbc

   DATA   hEnv, hDbc AS NUMERIC PROTECTED
   DATA   cConnect, cDSN, cUser, cPassword
   DATA   nRetCode

   METHOD New( cDSN, cUser, cPassword, cConnect, lDlg ) CONSTRUCTOR

   METHOD End()
   
   METHOD GetInfo( nType )
   
   METHOD SetOptions( nType, uBuffer )
   
   METHOD GetOptions( nType)
   
   METHOD Commit() 
   
   METHOD RollBack()

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( cDSN, cUser, cPassword, cConnect, lDlg ) CLASS TDbOdbc

   local hEnv, hDbc
   local nret

   ::nRetCode := 0

   DEFAULT lDlg := .f.           // Dialog Box connection

   ::cDSN      = cDSN
   ::cUser     = cUser
   ::cPassword = cPassword

   if ( nRet := SQLAllocEnv( @hEnv ) ) == SQL_SUCCESS
      ::hEnv = hEnv
   else
      ::nRetCode:=nRet
      MsgStop( "SQLALLOCENV Error" )
      return nil
   endif

   if (nret:=SQLAllocConnect( hEnv, @hDbc )) == SQL_SUCCESS
      ::hDbc = hDbc
   else
      ::nRetCode:=nRet
      MsgStop( "SQLALLOCCONNECT Error" )
      return nil
   endif

   if ! Empty( cConnect )
      if ( nRet := SQLDriverConnect( hDbc, @cConnect, lDlg ) ) != SQL_SUCCESS
         ::nRetCode = nRet
         MsgStop( "SQLDRIVERCONNECT Error: Not conexion established ODBC" )
         return nil
      else
         ::cConnect = cConnect
         ::cDSN     = SubStr( cConnect, 1, At( ";", cConnect ) - 1 )
         ::cDSN     = SubStr( ::cDSN, At( "=", ::cDSN ) + 1 )
      endif
   else
      nRet = SQLConnect( hDbc, cDSN, cUser, cPassword )
      if nret == SQL_SUCCESS .or. nret == SQL_SUCCESS_WITH_INFO
         //MSGALERT("Establecida conexion")
      else
         ::nRetCode = nRet
         MsgStop( "SQLCONNECT Error: Not conexion ODBC established" )
         return nil
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD End() CLASS TDbOdbc

   SQLDisconnect( ::hDbc )
   SQLFreeConnect( ::hDbc )
   SQLFreeEnv( ::hEnv )

   ::hEnv  = 0
   ::hDbc  = 0

return nil

//----------------------------------------------------------------------------//

METHOD GetInfo ( nType ) CLASS TDbOdbc

   local cBuffer := Space( 256 )
   
   ::nRetCode := SQLGetInfo( ::hDbc, nType, @cBuffer )

return cBuffer

//----------------------------------------------------------------------------//

METHOD GetOptions ( nType ) CLASS TDbOdbc

   local cBuffer:=space(256)
   
   ::nRetCode := SQLGetConn( ::hDbc, nType, @cBuffer )

return cBuffer

//----------------------------------------------------------------------------//

METHOD SetOptions ( nType, uBuffer ) CLASS TDbOdbc

return ( ::nRetCode := SQLSetConn( ::hDbc, nType, uBuffer ) )

//----------------------------------------------------------------------------//

METHOD Commit ( ) CLASS TDbOdbc
  
return ( ::nRetCode := SQLCommit( ::hEnv, ::hDbc ) )

//----------------------------------------------------------------------------//

METHOD RollBack() CLASS TDbOdbc
  
return ( ::nRetCode := SQLRollBac( ::hEnv, ::hDbc ) )

//----------------------------------------------------------------------------//
