#include "FiveWin.ch"
#include "Constant.ch"

#define COLOR_WINDOW         5
#define COLOR_WINDOWTEXT     8

#define FD_BORDER            8
#define FD_HEIGHT           22

#define DT_CENTER            1

//----------------------------------------------------------------------------//

CLASS TFolder FROM TControl

   CLASSDATA lRegistered AS LOGICAL

   DATA   aPrompts, aDialogs
   DATA   nOption, nTabSize
   DATA   oFont2

   METHOD New( nTop, nLeft, aPrompts, aDialogs, oWnd, nOption, nClrFore,;
               nClrBack, lPixel, lDesign, nWidth, nHeight,;
               cMsg ) CONSTRUCTOR

   METHOD ReDefine( nId, aPrompts, aDialogs, oWnd, nOption, nClrFore,;
                    nClrBack ) CONSTRUCTOR

   METHOD Display()
   METHOD Paint()
   METHOD Init( hDlg )
   METHOD LButtonDown( nRow, nCol, nFlags )
   METHOD Default()
   METHOD ReSize( nType, nWidth, nHeight )

   METHOD AddItem( cItem )

   METHOD DelItem()

   METHOD Destroy()

   METHOD GotFocus()

   METHOD SetOption( nOption )

   METHOD GetHotPos( nChar )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( nTop, nLeft, aPrompts, aDialogs, oWnd, nOption, nClrFore,;
            nClrBack, lPixel, lDesign, nWidth, nHeight, cMsg ) CLASS TFolder

   DEFAULT aPrompts := { "&One", "&Two", "T&hree" },;
           aDialogs := Array( Len( aPrompts ) ),;
           nOption  := 1,;
           nClrFore := oWnd:nClrText,;
           nClrBack := CLR_LIGHTGRAY,;
           lPixel   := .f.,;
           lDesign  := .f.,;
           nWidth   := 300, nHeight := 200

   ::nStyle    = nOR( WS_CHILD, WS_VISIBLE,;
                      If( lDesign, WS_CLIPSIBLINGS, 0 ),;
                      WS_TABSTOP )
   ::nId       = ::GetNewId()
   ::oWnd      = oWnd
   ::aPrompts  = aPrompts
   ::aDialogs  = aDialogs
   ::nOption   = nOption
   ::cMsg      = cMsg
   ::nTop      = If( lPixel, nTop, nTop * SAY_CHARPIX_H )
   ::nLeft     = If( lPixel, nLeft, nLeft * SAY_CHARPIX_W )
   ::nBottom   = ::nTop + nHeight - 1
   ::nRight    = ::nLeft + nWidth - 1
   ::lDrag     = lDesign
   ::lCaptured = .f.
   ::oFont     = TFont():New( "Ms Sans Serif", 0, -9 )
   ::oFont2    = TFont():New( "Ms Sans Serif", 0, -9,, .t. )

   ::SetColor( nClrFore, nClrBack )

   ::Register()

   if ! Empty( oWnd:hWnd )
      ::Create()
      oWnd:AddControl( Self )
   else
      oWnd:DefControl( Self )
   endif

   ::Default()

return nil

//----------------------------------------------------------------------------//

METHOD ReDefine( nId, aPrompts, aDialogs, oWnd, nOption, ;
                 nClrFore, nClrBack) CLASS TFolder

   local n, oDlg

   DEFAULT nOption  := 1,;
           nClrFore := oWnd:nClrText,;
           nClrBack := oWnd:nClrPane,;
           aDialogs := Array( Len( aPrompts ) )

   ::nId      = nId
   ::oWnd     = oWnd
   ::aPrompts = aPrompts
   ::aDialogs = aDialogs
   ::nOption  = nOption
   ::oFont    = TFont():New( "Ms Sans Serif", 0, -9 )
   ::oFont2   = TFont():New( "Ms Sans Serif", 0, -9,, .t. )

   ::Register()

   ::SetColor( nClrFore, nClrBack )

   for n = 1 to Len( ::aDialogs )
      DEFINE DIALOG oDlg OF Self RESOURCE ::aDialogs[ n ]
      ::aDialogs[ n ] = oDlg
   next

   oWnd:DefControl( Self )

return nil

//----------------------------------------------------------------------------//

METHOD Display() CLASS TFolder

   ::BeginPaint()
   ::Paint()
   ::EndPaint()

return 0

//----------------------------------------------------------------------------//

METHOD Paint() CLASS TFolder

   FoldPaint( ::hWnd, ::hDC, ::aPrompts, ::oFont:hFont,;
              ::oFont2:hFont, ::nClrPane, ::nOption )

return nil

//----------------------------------------------------------------------------//

METHOD Init( hDlg ) CLASS TFolder

   Super:Init( hDlg )

   ::Default()

return nil

//----------------------------------------------------------------------------//

METHOD LButtonDown( nRow, nCol, nFlags ) CLASS TFolder

   local n := 1, nPos := 0

   if ::lDrag
      return Super:LButtonDown( nRow, nCol, nFlags )
   else
      if nRow <= FD_HEIGHT
         while nCol > nPos + ::nTabSize
            nPos += ::nTabSize
            n++
         end
         ::SetOption( n )
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Default() CLASS TFolder

   local nLen := Len( ::aPrompts )
   local oDlg

   if nLen > 0
      ::nTabSize = ( ::nRight - ::nLeft + 1 ) / nLen
   else
      ::nTabSize = ::nRight - ::nLeft + 1
   endif

   for nLen = 1 to Len( ::aDialogs )
      oDlg = ::aDialogs[ nLen ]

      ACTIVATE DIALOG oDlg NOWAIT ;
         ON INIT oDlg:Move( FD_HEIGHT + 2, 3 ) ;
         VALID .f.                // to avoid exiting pressing Esc !!!

      oDlg:Hide()
   next

   if Len( ::aDialogs ) > 0
      if ::nOption <= Len( ::aDialogs )
         ::aDialogs[ ::nOption ]:Show()
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Resize( nType, nWidth, nHeight ) CLASS TFolder

   local nLen := Len( ::aPrompts )

   if nLen > 0
      ::nTabSize = nWidth / nLen
   else
      ::nTabSize = ::nRight - ::nLeft + 1
   endif

return Super:Resize( nType, nWidth, nHeight )

//----------------------------------------------------------------------------//

METHOD AddItem( cItem ) CLASS TFolder

   AAdd( ::aPrompts, cItem )
   ::Default()
   ::Refresh()

return nil

//----------------------------------------------------------------------------//

METHOD DelItem() CLASS TFolder

   if Len( ::aPrompts ) > 1
      ::aPrompts = ADel( ::aPrompts, ::nOption )
      ::aPrompts = ASize( ::aPrompts, Len( ::aPrompts ) - 1 )
   else
      ::aPrompts = { "No Defined" }
   endif
   ::nOption = Min( ::nOption, Len( ::aPrompts ) )
   ::Default()
   ::Refresh()

return nil

//----------------------------------------------------------------------------//

METHOD Destroy() CLASS TFolder

   local n

   ::oFont2:End()

   for n = 1 to Len( ::aDialogs )
      ::aDialogs[ n ]:bValid = { || .t. }
      ::aDialogs[ n ]:End()
      SysRefresh()
   next

return Super:Destroy()

//----------------------------------------------------------------------------//

METHOD GotFocus() CLASS TFolder

   Super:GotFocus()

   if ::nOption <= Len( ::aDialogs )
      ::aDialogs[ ::nOption ]:SetFocus()
   endif

return 0

//----------------------------------------------------------------------------//

METHOD SetOption( nOption ) CLASS TFolder

   if nOption > 0 .and. nOption != ::nOption
      if ::nOption <= Len( ::aDialogs ) .and. ::aDialogs[ ::nOption ] != nil
         ::aDialogs[ ::nOption ]:Hide()
      endif
      ::nOption = nOption
      InvalidateRect( ::hWnd, { 0, 0, FD_HEIGHT + 2, ::nRight } )
      if nOption <= Len( ::aDialogs ) .and. ::aDialogs[ nOption ] != nil
         ::aDialogs[ nOption ]:Show()
         ::aDialogs[ nOption ]:SetFocus()
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GetHotPos( nChar ) CLASS TFolder

   local n := 1
   local nAt

   while n <= Len( ::aPrompts )
     if ( nAt := At( "&", ::aPrompts[ n ] ) ) != 0 .and. ;
        Lower( SubStr( ::aPrompts[ n ], nAt + 1, 1 ) ) == Chr( nChar )
        return n
     endif
     n++
   end

return 0

//----------------------------------------------------------------------------//
