#include "FiveWin.ch"

#define FW_NORMAL          400
#define FW_BOLD            700

#define OEM_FIXED_FONT      10
#define ANSI_FIXED_FONT     11
#define ANSI_VAR_FONT       12
#define SYSTEM_FONT         13
#define DEVICE_DEFAULT_FONT 14
#define SYSTEM_FIXED_FONT   16

#define MM_ANISOTROPIC       8

//----------------------------------------------------------------------------//

CLASS TFont

   DATA   cFaceName
   DATA   hFont, hFontOld
   DATA   lBold, lItalic, lUnderline, lStrikeOut
   DATA   nHeight, nWidth, nEscapement, nOrientation, nWeight
   DATA   nCharSet, nOutPrecision, nClipPrecision, nQuality, nPitchFamily
   DATA   nCount
   DATA   lDestroy

   METHOD New( cFaceName, nWidth, nHeight, lFromUser,;
               lBold, nEscapement, nOrientation, nWeight, lItalic,;
               lUnderline, lStrikeOut, nCharSet, nOutPrecision,;
               nClipPrecision, nQuality, oDevice ) CONSTRUCTOR

   METHOD Choose( nRGBColor )     // passed by reference

   METHOD End()

   METHOD Release() INLINE ::End()

   METHOD Activate( hDC ) INLINE ::hFontOld := SelectObject( hDC, ::hFont )

   METHOD DeActivate( hDC ) INLINE  SelectObject( hDC, ::hFontOld )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( cFaceName, nWidth, nHeight, lFromUser, lBold,;
            nEscapement, nOrientation, nWeight, lItalic, lUnderline,;
            lStrikeOut, nCharSet, nOutPrecision, nClipPrecision,;
            nQuality, oDevice ) CLASS TFont

   local aFont, nRGBColor
   local nAt, aSystem := { "OEM_FIXED_FONT", "ANSI_FIXED_FONT",;
                           "ANSI_VAR_FONT", "SYSTEM_FONT",;
                           "DEVICE_DEFAULT_FONT", "SYSTEM_FIXED_FONT" }
   local aInfo, aSysFonts := { OEM_FIXED_FONT, ANSI_FIXED_FONT,;
                               ANSI_VAR_FONT, SYSTEM_FONT,;
                               DEVICE_DEFAULT_FONT, SYSTEM_FIXED_FONT }

   DEFAULT cFaceName := "", nEscapement := 0, nOrientation := 0, nWeight := 0,;
           nHeight := 14, lItalic := .f., lFromUser := .f., lUnderLine := .f.,;
           lStrikeOut := .f., nCharSet := 0, nOutPrecision := 0,;
           nClipPrecision := 0, nQuality := 0, lBold := .f.

   if lFromUser
      aFont     = ChooseFont( , @nRGBColor )
      lFromUser = ! Empty( aFont[ LF_FACENAME ] )
      aFont[ LF_HEIGHT ] = Int( aFont[ LF_HEIGHT ] * 10 / 13 )
   endif

   if lBold
      nWeight = nOr( nWeight, FW_BOLD )
   endif

   ::cFaceName      = If( lFromUser, aFont[ LF_FACENAME ], cFaceName )
   ::nHeight        = If( lFromUser, aFont[ LF_HEIGHT ], nHeight )
   ::nWidth         = If( lFromUser, aFont[ LF_WIDTH ], nWidth )
   ::nEscapement    = If( lFromUser, aFont[ LF_ESCAPEMENT ], nEscapement )
   ::nOrientation   = If( lFromUser, aFont[ LF_ORIENTATION ], nOrientation )
   ::nWeight        = If( lFromUser, aFont[ LF_WEIGHT ], nWeight )
   ::lBold          = If( lFromUser, ! ( ::nWeight == FW_NORMAL ), lBold )
   ::lItalic        = If( lFromUser, aFont[ LF_ITALIC ], lItalic )
   ::lUnderLine     = If( lFromUser, aFont[ LF_UNDERLINE ], lUnderLine )
   ::lStrikeOut     = If( lFromUser, aFont[ LF_STRIKEOUT ], lStrikeOut )
   ::nCharSet       = If( lFromUser, aFont[ LF_CHARSET ], nCharSet )
   ::nOutPrecision  = If( lFromUser, aFont[ LF_OUTPRECISION ], nOutPrecision )
   ::nClipPrecision = If( lFromUser, aFont[ LF_CLIPPRECISION ], nClipPrecision )
   ::nQuality       = If( lFromUser, aFont[ LF_QUALITY ], nQuality )
   ::nPitchFamily   = If( lFromUser, aFont[ LF_PITCHANDFAMILY ], 0 )
   ::hFontOld       = 0
   ::nCount         = 0

   if oDevice != nil // .and. GetMapMode( oDevice:hDC ) == MM_ANISOTROPIC
      ::nHeight = ( oDevice:nLogPixelY() / 72 ) * ::nHeight
   endif

   if ( nAt := AScan( aSystem, { | cType | cType == Upper( ::cFaceName ) } ) ) != 0
      ::hFont = GetStockObject( aSysFonts[ nAt ] )
      aInfo = GetFontInfo( ::hFont )
      ::nHeight  = aInfo[ 1 ]
      ::nWidth   = aInfo[ 2 ]
      ::lBold    = aInfo[ 3 ]
      ::lDestroy = .f.
   else
      ::hFont = CreateFont( { ::nHeight, ::nWidth, ::nEscapement,;
                              ::nOrientation, ::nWeight, ::lItalic,;
                              ::lUnderLine, ::lStrikeOut, ::nCharSet,;
                              ::nOutPrecision, ::nClipPrecision,;
                              ::nQuality, ::nPitchFamily, ::cFaceName } )
      ::lDestroy = .t.
   endif

   if ::nHeight < 0
      ::nHeight = Abs( ::nHeight )
      ::nWidth  = ::nHeight * 0.44
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Choose( nRGBColor ) CLASS TFont

   local aFont := ChooseFont( { ::nHeight, ::nWidth, ::nEscapement,;
                                ::nOrientation, ::nWeight, ::lItalic,;
                                ::lUnderLine, ::lStrikeOut, ::nCharSet,;
                                ::nOutPrecision, ::nClipPrecision,;
                                ::nQuality, ::nPitchFamily, ::cFaceName },;
                               @nRGBColor )

   ::cFaceName      = aFont[ LF_FACENAME ]
   ::nHeight        = aFont[ LF_HEIGHT ] // * 10/13
   ::lBold          = ! ( ::nWeight == FW_NORMAL )
   ::nWidth         = aFont[ LF_HEIGHT ] * 0.44
   ::nEscapement    = aFont[ LF_ESCAPEMENT ]
   ::nOrientation   = aFont[ LF_ORIENTATION ]
   ::nWeight        = aFont[ LF_WEIGHT ]
   ::lItalic        = aFont[ LF_ITALIC ]
   ::lUnderLine     = aFont[ LF_UNDERLINE ]
   ::lStrikeOut     = aFont[ LF_STRIKEOUT ]
   ::nCharSet       = aFont[ LF_CHARSET ]
   ::nOutPrecision  = aFont[ LF_OUTPRECISION ]
   ::nClipPrecision = aFont[ LF_CLIPPRECISION ]
   ::nQuality       = aFont[ LF_QUALITY ]
   ::nPitchFamily   = aFont[ LF_PITCHANDFAMILY ]

   if ::cFaceName != "System"
      ::nCount = 0
      if ! Empty( ::hFont ) .and. ::lDestroy
         DeleteObject( ::hFont )
      endif
      ::hFont = CreateFont( { ::nHeight, ::nWidth, ::nEscapement,;
                              ::nOrientation, ::nWeight, ::lItalic,;
                              ::lUnderLine, ::lStrikeOut, ::nCharSet,;
                              ::nOutPrecision, ::nClipPrecision,;
                              ::nQuality, ::nPitchFamily, ::cFaceName } )
      ::lDestroy = .t.
   endif

return Self            //  returns a reference to the Object

//----------------------------------------------------------------------------//

METHOD End() CLASS TFont

   // Now different objects may use the same font!!!

   if ::nCount == nil
      ::nCount = 1
   endif

   if --::nCount < 1
      if ! Empty( ::hFont ) .and. ::lDestroy
         DeleteObject( ::hFont )
         ::hFont  = 0
         ::nCount = 0
      endif
   endif

return nil

//----------------------------------------------------------------------------//
