#include "FiveWin.ch"
#include "Constant.ch"

#define SC_RESTORE     61728
#define SC_MINIMIZE    61472
#define SC_CLOSE       61536
#define SC_NEXT        61504

#define SIZE_MAXIMIZED     2
#define CW_USEDEFAULT  32768

#define WM_GETFONT      49  // 0x0031

#define BAR_TOP         1
#define BAR_LEFT        2
#define BAR_RIGHT       3
#define BAR_DOWN        4
#define BAR_FLOAT       5

static oWndMain

//----------------------------------------------------------------------------//

CLASS TMdiFrame FROM TWindow

   CLASSDATA lRegistered AS LOGICAL

   DATA   oWndClient, oWndActive
   DATA   oMenuStart
   DATA   nMenuInfo

   METHOD New( nTop, nLeft, nBottom, nRight, cTitle, nStyle, oMenu,;
               oBrush, oIcon, nClrFore, nClrBack, lVScroll,;
               lHScroll, nMenuInfo, cBorder, oWnd ) CONSTRUCTOR

   METHOD ChildNew( nTop, nLeft, nBottom, nRight, cTitle, nStyle ) INLINE ;
          ::oWndClient:ChildNew( nTop, nLeft, nBottom, nRight, cTitle, nStyle )

   METHOD Cascade()      INLINE ::oWndClient:Cascade()
   METHOD CloseAll()     INLINE ::oWndClient:lCloseAll()
   METHOD IconizeAll()   INLINE ::oWndClient:IconizeAll()

   METHOD Tile( lHor )   INLINE ::oWndClient:Tile( lHor )
   METHOD ArrangeIcons() INLINE ::oWndClient:ArrangeIcons()
   METHOD NextWindow()   INLINE ::oWndClient:NextWindow()

   METHOD ReSize( nSizeType, nWidth, nHeight )

   METHOD Command( nWParam, nLParam )

   METHOD End()

   METHOD Select( nWindow ) INLINE ::oWndClient:Select( nWindow )

   METHOD SetMenu( oMenu, nMenuInfo )

   METHOD GotFocus()

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( nTop, nLeft, nBottom, nRight, cTitle, nStyle, oMenu, oBrush,;
            oIcon, nClrFore, nClrBack, lVScroll, lHScroll, nMenuInfo,;
            cBorder, oWnd ) CLASS TMdiFrame

   DEFAULT nStyle    := nOR( WS_OVERLAPPEDWINDOW, WS_CLIPCHILDREN ),;
           lVScroll  := .f., lHScroll := .f.,;
           nMenuInfo := If( oMenu != nil, Len( oMenu:aItems ), 2 ),;
           nClrFore  := CLR_BLACK, nClrBack := CLR_GRAY

   ::nTop        = If( nTop    != nil, nTop    * MDIF_CHARPIX_H, CW_USEDEFAULT )			//16
   ::nLeft       = If( nLeft   != nil, nLeft   * MDIF_CHARPIX_W, CW_USEDEFAULT )			// 8
   ::nBottom     = If( nBottom != nil, nBottom * MDIF_CHARPIX_H, CW_USEDEFAULT )		//16
   ::nRight      = If( nRight  != nil, nRight  * MDIF_CHARPIX_W, CW_USEDEFAULT )		//  8
   ::nStyle      = nStyle
   ::cCaption    = cTitle
   ::nId         = 0
   ::oIcon       = oIcon
   ::lValidating = .f.
   ::lVisible    = .t.
   ::oWnd        = oWnd
   ::oMenu       = oMenu
   ::nMenuInfo   = nMenuInfo

   if oMenu == nil
      MENU oMenu
         MENUITEM "&Childs"
         MENU
            MENUITEM "New..." ACTION ::ChildNew()
            SEPARATOR
            MENUITEM "E&xit"  ACTION ::End()
         ENDMENU
         oMenu:AddMdi()
         oMenu:AddHelp( "FiveWin", "(c) FiveTech" )

      ENDMENU
      ::oMenu = oMenu
   endif

   ::Register()
   ::Create()                   // After this the window is subclassed and
   ::nOldProc = GetFrameProc()  // we install the MdiFrame procedure now !!!

   ::SetColor( nClrFore, nClrBack, oBrush )
                                        // SubMenu, nChild
   ::oWndClient = TMdiClient():New( Self, nMenuInfo, 1,, lVScroll,;
                                    lHScroll )
   ::oWndClient:SetColor( nClrFore, nClrBack, oBrush )

   ::SetMenu( oMenu, nMenuInfo )

   if ::oFont == nil
      ::oFont = ::GetFont()
   endif

   oWndMain = Self
   SetWndDefault( Self )

return nil

//----------------------------------------------------------------------------//

METHOD ReSize( nSizeType, nWidth, nHeight ) CLASS TMdiFrame

   local nTop    := 0
   local nLeft   := 0

   if ::oWndClient != NIL

      nWidth  = ::GetCliRect():nRight
      nHeight = ::GetCliRect():nBottom - ;
                If( ::oMsgBar != nil, ::oMsgBar:Height(), 0 )

      if ::oBar != nil
         do case
            case ::oBar:nMode == BAR_TOP
                 nTop     = ( ::oBar:nBtnHeight + 1 )
                 nHeight -= ( ::oBar:nBtnHeight + 1 )   // 2

            case ::oBar:nMode == BAR_LEFT
                 nLeft    = ( ::oBar:nBtnWidth + 4 )
                 nWidth  -= ( ::oBar:nBtnWidth + 4 )

            case ::oBar:nMode == BAR_RIGHT
                 nWidth  -= ( ::oBar:nBtnWidth + 4 )

            case ::oBar:nMode == BAR_DOWN
                 nHeight -= ( ::oBar:nBtnHeight + 2 )
         endcase
      endif
      ::oWndClient:Move( nTop, nLeft, nWidth, nHeight, .t. )
   endif

   Super:ReSize( nSizeType, nWidth, nHeight )

return 0

//----------------------------------------------------------------------------//

METHOD Command( nWParam, nLParam ) CLASS TMdiFrame

   local oWnd := ::oWndClient:GetActive()

   do case
      case nWParam < 100   // MdiChild Menu Selection
           return nil

      otherwise
         Super:Command( nWParam, nLParam )
   endcase

return nil

//----------------------------------------------------------------------------//

METHOD End() CLASS TMdiFrame

   if ::bValid != nil
      if ! Eval( ::bValid )
         return .f.
      endif
   endif

   if ::oWndClient:lCloseAll()
      ::PostMsg( WM_CLOSE )
   else
      return .f.
   endif

return .t.

//----------------------------------------------------------------------------//

METHOD SetMenu( oMenu, nMenuInfo ) CLASS TMdiFrame

  DEFAULT nMenuInfo := Len( oMenu:aItems )

  ::nMenuInfo = nMenuInfo

  if ::oMenuStart == nil
     ::oMenuStart = oMenu
     Super:SetMenu( oMenu )
  else
     ::oMenu = oMenu
     oMenu:oWnd = Self
     ::oWndClient:SetMenu( oMenu, nMenuInfo )
  endif

return nil

//----------------------------------------------------------------------------//

function GetWndFrame() ; return oWndMain      // Default Frame Window

//----------------------------------------------------------------------------//

METHOD GotFocus() CLASS TMdiFrame

   ::lFocused = .t.

   if ::bGotFocus != nil
      Eval( ::bGotFocus )
   endif

return nil

//----------------------------------------------------------------------------//
