#include "FiveWin.ch"

#define MF_BYCOMMAND  0
#define MF_ENABLED    0
#define MF_UNCHECKED  0
#define MF_GRAYED     1
#define MF_DISABLED   2
#define MF_CHECKED    8
#define MF_POPUP     16   // 0x0010
#define MF_HELP   16384   // 0x4000

static nId := 100    // To avoid conflicts with MDI automatic MenuItems

//----------------------------------------------------------------------------//

CLASS TMenuItem

   DATA   cPrompt, cMsg
   DATA   nId, nHelpId
   DATA   bAction
   DATA   lChecked, lActive, lHelp
   DATA   oMenu
   DATA   hBitmap

   METHOD New( cPrompt, cMsg, lChecked, lActive, bAction, cBmpFile,;
               cResName, nVKState, nVirtKey, lHelp, nHelpId ) CONSTRUCTOR

   METHOD ReDefine( cPrompt, cMsg, lChecked, lActive, bAction, cBmpFile,;
                    cResName, oMenu, bBlockAction, nId, nVKState,;
                    nVirtKey, nHelpId ) CONSTRUCTOR

   METHOD SetCheck( lOnOff ) BLOCK ;
      { | Self, lOnOff | ::lChecked := lOnOff, CheckMenuItem( ::oMenu:hMenu,;
      ::nId, nOR( MF_BYCOMMAND, If( lOnOff, MF_CHECKED, MF_UNCHECKED ) ) ) }

   METHOD Enable() BLOCK ;
      { | Self | ::lActive := .t.,;
                 EnableMenuItem( ::oMenu:hMenu, ::nId, MF_ENABLED ) }

   MESSAGE HelpTopic METHOD __HelpTopic()

   METHOD Disable() INLINE ::lActive := .f.,;
      EnableMenuItem( ::oMenu:hMenu, ::nId, nOR( MF_DISABLED, MF_GRAYED ) ),;
      If( ValType( ::bAction ) == "O", ::bAction:Disable(),)

   METHOD End() INLINE If( ::hBitmap != 0, DeleteObject( ::hBitmap ),)

   METHOD SetPrompt( cPrompt )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( cPrompt, cMsg, lChecked, lActive, bAction, cBmpFile, cResName,;
            nVKState, nVirtKey, lHelp, nHelpId ) CLASS TMenuItem

   DEFAULT lChecked := .f., lActive := .t., cBmpFile := "",;
           lHelp := .f.

   ::bAction  = bAction
   ::cPrompt  = cPrompt
   ::cMsg     = cMsg
   ::nId      = nId++
   ::lChecked = lChecked
   ::lActive  = lActive
   ::lHelp    = lHelp
   ::hBitmap  = 0
   ::nHelpId  = nHelpId

   if ! Empty( cBmpFile ) .and. File( cBmpFile )
      ::hBitmap = ReadBitmap( 0, cBmpFile )
   endif
   if ! Empty( cResName )
      ::hBitmap = LoadBitmap( GetResources(), cResName )
   endif

   if nVirtKey != nil
      __AddAccel( nVKState, nVirtKey, ::nId )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD ReDefine( cPrompt, cMsg, lChecked, lActive, bAction, cBmpFile,;
                 cResName, oMenu, bBlockAction, nId, nVKState,;
                 nVirtKey, nHelpId ) CLASS TMenuItem

   DEFAULT oMenu := ::oMenu

   if oMenu != nil
      Self = oMenu:GetMenuItem( nId )
   endif

   if lChecked != nil
      if lChecked != ::lChecked
         ::lChecked = lChecked
         if oMenu != nil .and. nId != nil
            CheckMenuItem( oMenu:hMenu, nId, nOr( MF_BYCOMMAND,;
                           If( lChecked, MF_CHECKED, MF_UNCHECKED ) ) )
         endif
      endif
   endif

   if lActive != nil
      if lActive != ::lActive
         ::lActive = lActive
         if oMenu != nil .and. nId != nil
            EnableMenuItem( oMenu:hMenu, nId, nOr( MF_BYCOMMAND,;
                            If( lActive, MF_ENABLED, MF_DISABLED ) ) )
         endif
      endif
   endif

   ::cMsg     = cMsg
   ::bAction  = bAction
   ::oMenu    = oMenu
   ::hBitmap  = 0
   ::nHelpId  = nHelpId

   if bBlockAction != nil
      ::bAction = bBlockAction
   endif

   if ! Empty( cBmpFile )
      ::hBitmap = ReadBitmap( 0, cBmpFile )
   endif
   if ! Empty( cResName )
      ::hBitmap = LoadBitmap( GetResources(), cResName )
   endif

   if nVirtKey != nil
      __AddAccel( nVKState, nVirtKey, ::nId )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD SetPrompt( cPrompt ) CLASS TMenuItem

   local nNewId, nFlags

   ::cPrompt = cPrompt

   if ::bAction:ClassName() == "TMENU"
      nNewId := ::bAction:hMenu
      nFlags := MF_POPUP
   else
      nNewId := ::nId
      nFlags := nOR( If( ::lActive, MF_ENABLED,;
                nOR( MF_DISABLED, MF_GRAYED ) ),;
                If( ::lChecked, MF_CHECKED, 0 ),;
                If( ::lHelp, MF_HELP, 0 ) )
   endif

   ModifyMenu( ::oMenu:hMenu, ::nId, nFlags, nNewId, cPrompt )

   if ::oMenu:oWnd != nil
      DrawMenuBar( ::oMenu:oWnd:hWnd )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD __HelpTopic() CLASS TMenuItem

   if Empty( ::nHelpId )
      if ::oMenu != nil
         ::oMenu:HelpTopic()
      else
         HelpIndex()
      endif
   else
      HelpTopic( ::nHelpId )
   endif

return nil

//----------------------------------------------------------------------------//
