#include "FiveWin.ch"
#include "Constant.ch"
#include "Set.ch"

#define EM_GETSEL       (WM_USER+ 0)
#define EM_SETSEL       (WM_USER+ 1)
#define EM_GETLINECOUNT (WM_USER+10)
#define EM_LINEINDEX    (WM_USER+11)
#define EM_LINEFROMCHAR (WM_USER+25)

#define COLOR_WINDOW              5
#define COLOR_WINDOWTEXT          8

#define ES_CENTER                 1

#define WM_SETFONT               48

//----------------------------------------------------------------------------//

CLASS TMultiGet FROM TControl

   DATA   lReadOnly
   DATA   nPos

   METHOD New( nRow, nCol, bSetGet, oWnd, nWidth, nHeight, oFont,;
               lHScroll, nClrFore, nClrBack, oCursor, lPixel, cMsg,;
               lUpdate, bWhen, lCenter, lRight, lReadOnly, bValid,;
               bChanged ) CONSTRUCTOR

   METHOD ReDefine( nId, bSetGet, oWnd, nHelpId, nClrFore, nClrBack, oFont,;
                    oCursor, cMsg, lUpdate, bWhen, lReadOnly, bValid,;
                    bChanged ) CONSTRUCTOR

   METHOD cToChar() INLINE  Super:cToChar( "EDIT" )

   METHOD Copy()

   METHOD Default()

   METHOD GetLine( nLine ) INLINE MGetLine( ::hWnd, nLine )

   METHOD GetLineCount() INLINE SendMessage( ::hWnd, EM_GETLINECOUNT )

   METHOD GotFocus()

   METHOD GoTo( nLine ) INLINE ;
                ::SetPos( SendMessage( ::hWnd, EM_LINEINDEX, nLine - 1, 0 ) )

   METHOD Init( hDlg )

   METHOD LostFocus()

   METHOD MouseMove( nRow, nCol, nKeyFlags )

   METHOD Move( nTop, nLeft, nBottom, nRight, lRepaint )

   METHOD cText( cText ) SETGET

   METHOD Paste( cText )

   METHOD Refresh() INLINE ;
                    SetWindowText( ::hWnd, cValToChar( Eval( ::bSetGet ) ) )

   METHOD Save() INLINE Eval( ::bSetGet, GetWindowText( ::hWnd ) )

   METHOD SetCoors( oRect )

   METHOD VScroll( nWParam, nLParam ) VIRTUAL

   METHOD HScroll( nWParam, nLParam ) VIRTUAL

   METHOD GetSel()

   METHOD KeyChar( nKey, nFlags )

   METHOD KeyDown( nKey, nFlags )

   METHOD SetPos( nStart, nEnd ) INLINE ;
                 nEnd := If( nEnd == nil, nStart, nEnd ),;
                 ::SendMsg( EM_SETSEL, 0, nMakeLong( nStart, nEnd ) ),;
                 ::nPos := nStart

   METHOD GetRow()
   METHOD GetCol()

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( nRow, nCol, bSetGet, oWnd, nWidth, nHeight, oFont, lHScroll,;
            nClrFore, nClrBack, oCursor, lPixel, cMsg, lUpdate,;
            bWhen, lCenter, lRight, lReadOnly, bValid, bChanged ) CLASS TMultiGet

   DEFAULT lHScroll  := .f.,;
           nClrFore  := GetSysColor( COLOR_WINDOWTEXT ),;
           nClrBack  := GetSysColor( COLOR_WINDOW ),;
           lPixel    := .f., lUpdate := .f.,;
           lCenter   := .f., lRight := .f.,;
           lReadOnly := .f.

   ::cCaption = cValToChar( Eval( bSetGet ) )
   ::nTop     = nRow * If( lPixel, 1, MGET_CHARPIX_H )  //13
   ::nLeft    = nCol * If( lPixel, 1, MGET_CHARPIX_W )	// 8
   ::nBottom  = If( nHeight == nil, ::nTop + 11, ::nTop + nHeight )
   ::nRight   = If( nWidth == nil, ::nLeft + Len( ::cCaption ) * 3.5, ;
                    ::nLeft + nWidth )
   ::bSetGet  = bSetGet
   ::oWnd     = oWnd
   ::nStyle   = nOR( WS_CHILD, WS_VISIBLE, WS_BORDER, WS_TABSTOP, ES_LEFT,;
                     ES_WANTRETURN, ES_MULTILINE, WS_VSCROLL,;
                     If( lHScroll, WS_HSCROLL, 0 ),;
                     If( lCenter, ES_CENTER, If( lRight, ES_RIGHT, ES_LEFT ) ) )
   ::nId       = ::GetNewId()
   ::cCaption  = RTrim( ::cCaption )
   ::lDrag     = .f.
   ::lCaptured = .f.
   ::oCursor   = oCursor
   ::oFont     = oFont
   ::cMsg      = cMsg
   ::lUpdate   = lUpdate
   ::bWhen     = bWhen
   ::bValid    = bValid
   ::lReadOnly = lReadOnly
   ::nPos      = 0
   ::bChange   = bChanged

   ::SetColor( nClrFore, nClrBack )

   if ! Empty( oWnd:hWnd )
      ::Create( "EDIT" )
      PostMessage( ::hWnd, WM_SETFONT,;
                   If( oFont != nil, oFont:hFont, ::oWnd:oFont:hFont ) )
      oWnd:AddControl( Self )
   else
      oWnd:DefControl( Self )
   endif

   ::GetFont()

return nil

//----------------------------------------------------------------------------//

METHOD ReDefine( nId, bSetGet, oWnd, nHelpId, nClrFore, nClrBack, oFont,;
             oCursor, cMsg, lUpdate, bWhen, lReadOnly, bValid, bChanged ) CLASS TMultiGet

   DEFAULT nClrFore := GetSysColor( COLOR_WINDOWTEXT ),;
           nClrBack := GetSysColor( COLOR_WINDOW ),;
           lUpdate  := .f., lReadOnly := .f.

   ::nId       = nId
   ::bSetGet   = bSetGet
   ::oWnd      = oWnd
   ::nHelpId   = nHelpId
   ::lDrag     = .f.
   ::lCaptured = .f.
   ::oFont     = oFont
   ::oCursor   = oCursor
   ::cMsg      = cMsg
   ::lUpdate   = lUpdate
   ::bWhen     = bWhen
   ::bValid    = bValid
   ::lReadOnly = lReadOnly
   ::nPos      = 0
   ::bChange   = bChanged

   ::SetColor( nClrFore, nClrBack )

   oWnd:DefControl( Self )

return nil

//----------------------------------------------------------------------------//

METHOD Init( hDlg ) CLASS TMultiGet

   Super:Init( hDlg )
   SetWindowText( ::hWnd, cValToChar( Eval( ::bSetGet ) ) )
   ::Default()

return nil

//----------------------------------------------------------------------------//

METHOD cText( cText ) CLASS TMultiGet

   if PCount() == 1
      SetWindowText( ::hWnd, cText )
      Eval( ::bSetGet, cText )
   endif

return GetWindowText( ::hWnd )

//----------------------------------------------------------------------------//

METHOD Copy() CLASS TMultiGet

   local oClp

   DEFINE CLIPBOARD oClp OF Self ;
      FORMAT TEXT

   if oClp:Open()
      oClp:Clear()
      oClp:SetText( ::GetSel() )
      oClp:End()
   else
      MsgAlert( "The clipboard is not available now!" )
   endif

return nil

//---------------------------------------------------------------------------//

METHOD GetSel() CLASS TMultiGet

   local n      := ::SendMsg( EM_GETSEL )
   local nStart := nLoWord( n )
   local nEnd   := nHiWord( n )

return If( nStart != nEnd, SubStr( ::cText, nStart + 1, nEnd - nStart + 1 ), "" )

//----------------------------------------------------------------------------//

METHOD GetRow() CLASS TMultiGet

return ::SendMsg( EM_LINEFROMCHAR,;
                  nLoWord( ::SendMsg( EM_GETSEL ) ) ) + 1

//----------------------------------------------------------------------------//

METHOD GetCol() CLASS TMultiGet

   local nRow := ::GetRow()

return ::SendMsg( EM_LINEFROMCHAR, nRow )

//----------------------------------------------------------------------------//

METHOD MouseMove( nRow, nCol, nKeyFlags ) CLASS TMultiGet

   if ::lDrag
      return Super:MouseMove( nRow, nCol, nKeyFlags )
   else
      ::oWnd:SetMsg( ::cMsg )
      if ::oCursor != nil
         SetCursor( ::oCursor:hCursor )
      else
         CursorIBeam()
      endif

      ::ShowToolTip()

      if ::bMMoved != nil
         Eval( ::bMMoved, nRow, nCol, nKeyFlags )
      endif
   endif

return nil      // We want standard MultiLine Get behavior !!!

//---------------------------------------------------------------------------//

METHOD KeyDown( nKey, nFlags ) CLASS TMultiGet

   do case
      case nKey == VK_INSERT
           Set( _SET_INSERT, ! Set( _SET_INSERT ) )
           DestroyCaret()
           if Set( _SET_INSERT )
              CreateCaret( ::hWnd, 0, 6, ::nGetChrHeight() )
           else
              CreateCaret( ::hWnd, 0, 1, ::nGetChrHeight() )
           endif
           ShowCaret( ::hWnd )

      case nKey == VK_DELETE
           if ::lReadOnly
              return 0
           endif
   endcase

return Super:KeyDown( nKey, nFlags )

//---------------------------------------------------------------------------//

METHOD KeyChar( nKey, nFlags ) CLASS TMultiGet

   local bKeyAction := SetKey( nKey )
   local lAccept

   if bKeyAction != nil
      Eval( bKeyAction, ProcName( 4 ), ProcLine( 4 ) )
      return 0         // Already processed, API do nothing
   endif

   if ::lReadOnly
      return 0
   endif

   if ::bChange != nil
      lAccept = Eval( ::bChange, nKey, nFlags, Self )
      if ValType( lAccept ) == "L" .and. ! lAccept
         return 0
      endif
   endif

   if nKey == VK_RETURN
      return nil
   endif

return Super:KeyChar( nKey, nFlags )

//---------------------------------------------------------------------------//

METHOD Paste( cText ) CLASS TMultiGet

   local oClp
   local cTemp

   DEFINE CLIPBOARD oClp OF Self FORMAT TEXT

   if Empty( cText )
      if oClp:Open()
         cText = oClp:GetText()
         oClp:Close()
      else
         MsgAlert( "The clipboard is not available!" )
      endif
   endif

   if ! Empty( cText )
      cTemp = ::VarGet()
      ::VarPut( SubStr( cTemp, 1, ::nPos ) + cText + ;
                SubStr( cTemp, ::nPos + 1 ) )
      ::Refresh()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GotFocus() CLASS TMultiGet

   Super:GotFocus()

   ::SetPos( ::nPos )
   CallWindowProc( ::nOldProc, ::hWnd, WM_SETFOCUS )

   if Set( _SET_INSERT )
      DestroyCaret()
      CreateCaret( ::hWnd, 0, 6, ::nGetChrHeight() )
      ShowCaret( ::hWnd )
   endif

return 0

//----------------------------------------------------------------------------//

METHOD LostFocus() CLASS TMultiGet

   Super:LostFocus()
   Eval( ::bSetGet, GetWindowText( ::hWnd ) )
   ::nPos = nLoWord( ::SendMsg( EM_GETSEL ) )

return nil

//----------------------------------------------------------------------------//

METHOD Default() CLASS TGet

   if ::oFont != nil
      ::SetFont( ::oFont )
   else
      ::SetFont( ::oWnd:oFont )
   endif

return nil

//---------------------------------------------------------------------------//

METHOD Move( nTop, nLeft, nBottom, nRight, lRepaint ) CLASS TMultiGet

   Super:Move( nTop, nLeft, nBottom, nRight, lRepaint )
   MoveGet( ::hWnd, ::nRight - ::nLeft, ::nBottom - ::nTop )

return nil

//----------------------------------------------------------------------------//

METHOD SetCoors( oRect ) CLASS TMultiGet

   Super:SetCoors( oRect )
   MoveGet( ::hWnd, oRect:nRight - oRect:nLeft, oRect:nBottom - oRect:nTop )

return nil

//----------------------------------------------------------------------------//
