#include "FiveWin.ch"

#define COLOR_BTNFACE   15
#define COLOR_BTNSHADOW 16
#Define MSG_HEIGHT      23

//----------------------------------------------------------------------------//

CLASS TMsgBar FROM TWindow

   DATA   cMsgDef
   DATA   lCentered, lActive, lDate, lClock, lKbd, lInset
   DATA   oDate, oTime, oKbd, oFontClock, oFontDate, oFontKeyb
   DATA   aItem 
   DATA   nClrFore, nClrBack, nSizeItem 

   CLASSDATA lRegistered AS LOGICAL

   METHOD New( oWnd, cPrompt, lCentered, lClock, lDate, lKbd,;
               nClrFore, nClrBack, oFont, lInset ) CONSTRUCTOR

   METHOD AddItem( oItem ) INLINE oItem:nId := Len( ::aItem ) + 1,;
                                  AAdd( ::aItem, oItem ), ::Refresh( .t. )
 
   METHOD DelItem( oItem ) INLINE If( oItem:nId > 0,;
                                  ( ADel( ::aItem, oItem:nId ),;
                                  ASize( ::aItem, Len( ::aItem ) - 1 ),;
                                  oItem:nId := 0, ::Refresh( .t. ) ),)
                                  
   METHOD Adjust()

   METHOD ClockOn()
   METHOD ClockOff() INLINE If( ::lClock, ( ::lClock := .f.,;
                                ADel( ::aItem, 1 ),;
                                ASize( ::aItem, Len( ::aItem ) - 1 ),; 
                                ::oTime:End(), ::Refresh( .t. ) ),)

   METHOD DateOn()
   METHOD DateOff()  INLINE If( ::lDate, ( ::lDate := .f.,;
                               ADel( ::aItem, If( ::lClock, 2, 1 ) ),;
                               ASize( ::aItem, Len( ::aItem ) - 1 ),; 
                               ::oDate:End(), ::Refresh( .t. ) ),)

   METHOD Default()

   METHOD Destroy()

   METHOD GetItem()  

   METHOD Height() INLINE MSG_HEIGHT

   METHOD KeybOn()   
   METHOD KeybOff()  INLINE If(::lKbd, ( ::lKbd := .f.,;
                               ADel( ::aItem, GetPos( ::lClock, ::lDate ) ),;
                               ADel( ::aItem, GetPos( ::lClock, ::lDate ) ),;
                               ADel( ::aItem, GetPos( ::lClock, ::lDate ) ),;
                               ASize( ::aItem, Len( ::aItem ) - 3 ),; 
                               ::oKbd:End(), ::Refresh( .t. ) ),)   
                                              

   METHOD Paint() INLINE MsgPaint( ::hWnd, ::cMsg, ::cMsgDef, .t.,;
                                   ::lCentered, GetSizeItem( ::aItem ),;
                                   ::nClrFore, ::nClrBack, ::oFont:hFont,;
                                   ::lInset ),;
                                   ShowItem( Self, ::aItem, ::nRight )

   METHOD Refresh( lChange )
   METHOD SetMsg( cText )
  
   METHOD ShowMsgBar() VIRTUAL

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( oWnd, cPrompt, lCentered, lClock, lDate, lKbd,;
            nClrFore, nClrBack, oFont, lInset ) CLASS TMsgBar

   local oRect := oWnd:GetCliRect()

   DEFAULT cPrompt := "", lCentered := .f., lClock := .f., lDate := .f.,;
           lKbd := .f., nClrFore := CLR_BLACK, nClrBack := CLR_HGRAY,;
           lInset := .t.   
   
   ::nClrPane    = COLOR_BTNFACE + 1
   ::nClrFore    = nClrFore
   ::nClrBack    = nClrBack
   ::oWnd        = oWnd
   ::cMsg        = ::cMsgDef  := cPrompt
   ::nStyle      = nOR( WS_VISIBLE, WS_CHILD, WS_OVERLAPPED, WS_BORDER )
   ::nLeft       = -1
   ::nTop        = oRect:nBottom - 23
   ::nBottom     = oRect:nBottom
   ::nRight      = oRect:nRight  + 1
   ::oBrush      = TBrush():New( , nRGB( 192, 192, 192 ), )
   ::lCentered   = lCentered
   ::lValidating = .f.
   ::aItem       = {}
   ::lDate       = lDate
   ::lClock      = lClock 
   ::lKbd        = lKbd
   ::oFont       = oFont
   ::lInset      = lInset
   
   ::Register()
   
   ::Create()
   ::Default()

   If ::lClock
      ::ClockOn()
   Endif

   If ::lDate   
      ::DateOn()
   Endif

   If ::lKbd   
      ::KeybOn() 
   Endif 

return nil

//----------------------------------------------------------------------------//

METHOD Adjust() CLASS TMsgBar

   local rctParent := ::oWnd:GetCliRect()

   ::SetCoors( TRect():New( rctParent:nBottom - 23, rctParent:nLeft - 1,;
                            rctParent:nBottom, rctParent:nRight ) )
return nil

//----------------------------------------------------------------------------//

METHOD ClockOn() CLASS TMsgBar
 
   If ::oTime != Nil
      If ::oTime:lActive 
         Return nil
      Endif
   Endif
    
   ::lClock := .t. 
   ASize( ::aItem, Len( ::aItem ) + 1 )
   AIns( ::aItem, 1 )
   ::aItem[ 1 ] := TMsgItem():New( Self,, 54 )
   ::SetMsg()
   If ::oTime != Nil
      ::oTime:Activate()

   Else
      DEFINE FONT ::oFontClock NAME "Ms Sans Serif" SIZE 0, -8 

      DEFINE TIMER ::oTime OF Self INTERVAL 100;
             ACTION ::Say( 4, ::nRight - 52, Time(), ::nClrFore,;
                    ::nClrBack, ::oFontClock, .t.)

      ACTIVATE TIMER ::oTime       
 
   Endif

   ::Refresh( .t. )

return nil

//---------------------------------------------------------------------------//

METHOD DateOn() CLASS TMsgBar
 
   local nPos := If( ::lClock, 2, 1 )

   if ::oDate != nil
      if ::oDate:lActive
         return nil
      endif
   endif
   
   ::lDate := .t.
 
   ASize( ::aItem, Len( ::aItem ) + 1 )
   AIns( ::aItem, nPos )
   ::aItem[ nPos ] = TMsgItem():New( Self,, 56 )
   ::SetMsg() 
   if ::oDate != nil
      ::oDate:Activate()
   else
 
      DEFINE TIMER ::oDate OF Self INTERVAL 17;
             ACTION ::Say( 4, ::nRight - If( ::lClock, 112, 53 ),;
                    Date(), ::nClrFore, ::nClrBack, ::oFontDate, .t. )
      ACTIVATE TIMER ::oDate       

   Endif

   ::Refresh( .t. )

return nil

//----------------------------------------------------------------------------//

METHOD Default() CLASS TMsgBar
  
   if ::oFont != Nil
      ::SetFont( ::oFont )
   else
      DEFINE FONT ::oFont NAME "Ms Sans Serif" SIZE 0, -8
   endif

   DEFINE FONT ::oFontClock NAME "Ms Sans Serif" SIZE 0, -8
 
   ::oFontKeyb := ::oFontDate := ::oFontClock
 
   if Valtype( ::nClrFore ) == "C"
      ::nClrBack := ::nClrFore
      ::nClrFore := nGetForeRGB( ::nClrFore )
      ::nClrBack := nGetBackRGB( ::nClrBack )
   Endif
   
return nil

//---------------------------------------------------------------------------//

METHOD Destroy() CLASS TMsgBar

   ::ClockOff()
   ::DateOff()
   ::keybOff()
   ::oFontClock:End()

   Super:Destroy()

return nil

//----------------------------------------------------------------------------//

METHOD GetItem()

return Len( ::aItem )

//----------------------------------------------------------------------------//

METHOD KeybOn() CLASS TMsgBar

   local nCount, nPos

   if ::oKbd != nil
      if ::oKbd:lActive 
         return nil
      endif   
   endif

   ::lKbd := .t. 
   ASize( ::aItem, Len( ::aItem ) + 3 )
 
   nPos := GetPos( ::lClock, ::lDate )          
   for nCount := nPos to nPos + 2
      AIns( ::aItem, nCount )
      ::aItem[ nCount ] := TMsgItem():New( Self )
   next
 
   ::aItem[ nPos ]:nSize = 26
   ::aItem[ nPos + 1 ]:nSize = 34
   ::aItem[ nPos + 2 ]:nSize = 30
   ::SetMsg()

   if ::oKbd != nil
      ::oKbd:Activate()
   else

     DEFINE TIMER ::oKbd OF Self INTERVAL 17;
            ACTION ( ::Say(4, ::nRight - GetKeybPos( 1, ::lclock, ::lDate ), "Num",;
                    If( GetKeyToggle( VK_NUMLOCK ), CLR_GRAY, CLR_BLACK ),;
                       ::nClrBack, ::oFontKeyb, .t.),;
                    ::Say( 4, ::nRight - GetKeybPos( 2, ::lclock, ::lDate ), "Caps",;
                    If( GetKeyToggle( VK_CAPITAL ), CLR_GRAY, CLR_BLACK ),;
                       ::nClrBack, ::oFontKeyb, .t. ),;
                    ::Say( 4, ::nRight - GetKeybPos( 3, ::lClock, ::lDate ), "Ins",;
                    If( GetKeyToggle( VK_INSERT ), CLR_GRAY, CLR_BLACK ),;
                       ::nClrBack, ::oFontKeyb, .t. ) )

      ACTIVATE TIMER ::oKbd
   endif       
   
   ::Refresh(.t.)

return nil

//---------------------------------------------------------------------------//

METHOD Refresh( lChange ) CLASS TMsgBar
  
   DEFAULT lChange := .f.

   if lChange      
      AEval( ::aItem, {| oItem, nPos | oItem:nId := nPos } )
   endif   
   Super:Refresh() 

return nil

//---------------------------------------------------------------------------//

METHOD SetMsg( cText ) CLASS TMsgBar

   DEFAULT ::cMsg := ""

   if Empty( cText )
      if ! Empty( ::cMsg )
         MsgPaint( ::hWnd, ::cMsg := "", ::cMsgDef, .f.,;
                   ::lCentered, GetSizeItem( ::aItem ),;
                   ::nClrFore, ::nClrBack, ::oFont:hFont )
      endif
   else
      if ::cMsg != cText
         MsgPaint( ::hWnd, ::cMsg := cText, ::cMsgDef, .f.,;
                   ::lCentered, GetSizeItem( ::aItem ),;
                   ::nClrFore, ::nClrBack, ::oFont:hFont )
      endif
   endif

return nil

//----------------------------------------------------------------------------//

static function GetKeybPos( nPos, lClock, lDate )

   local aPos := Array( 3 )

   do case
      case lClock .and. lDate
         aPos[ 1 ] = 215                  
         aPos[ 2 ] = 179                  
         aPos[ 3 ] = 140                  

      Case lClock .or. lDate
         aPos[ 1 ] = 157                  
         aPos[ 2 ] = 120                  
         aPos[ 3 ] = 82                  
         
      Case ! lClock .and. ! lDate
         aPos[ 1 ] = 96                  
         aPos[ 2 ] = 62                  
         aPos[ 3 ] = 22                  
   endcase

return aPos[ nPos ]

//---------------------------------------------------------------------------//

static function GetSizeItem( aItem )

   Local nCount, nSize

   nSize := 5
   if empty( aItem )
      return nSize
   endif    

   for nCount := 1 to Len( aItem )
      nSize += aItem[ nCount ]:nSize + 4
   next

return nSize

//---------------------------------------------------------------------//

static function ShowItem( oWnd, aItem, nRight )

   Local nCount, nPos, hDC
   Local oRect, oBrush 

   hDC := GetDC( oWnd:hWnd )
   nPos := nRight

   for nCount := 1 to Len(aItem)

      nPos -= aItem[nCount]:nSize 
      WndBoxIn(hDC, 3, nPos, 18, (nPos + aItem[nCount]:nSize) - 4)    
      oRect:= TRect():New(4, nPos + 1, 17, nPos + (aItem[nCount]:nSize - 5))

      if ValType(aItem[ nCount ]:nClrFore ) == "C"
         aItem[ nCount ]:nClrBack := nGetBackRGB( aItem[ nCount ]:nClrFore )
      endif   

      If aItem[ nCount ]:cMsg != Nil
         DrawMsgItem( hDC, aItem[ nCount ]:cMsg, oRect,;
                  aItem[ nCount ]:nClrFore, aItem[ nCount ]:nClrBack,;
                  aItem[ nCount ]:oFont:hFont ) 
      else
         oBrush := TBrush():New( , aItem[ nCount ]:nClrBack )
         FillRect( hDC, oRect, oBrush:hBrush )
         oBrush:End()       
      endif
 
      nPos -= 4
   next

   ReleaseDC( oWnd:hWnd, hDC )

return nil

//------------------------------------------------------------------------//                

static function GetPos( lClock, lDate )

   local nPos

   do case
      case lClock .and. lDate
         nPos = 3

      case lClock .or. lDate
         nPos = 2

      case ! lClock .and. ! lDate
         nPos = 1
   endcase

return nPos

//------------------------------------------------------------------------//
