#include "FiveWin.ch"
#include "sql.ch"
#include "Set.ch"

//----------------------------------------------------------------------------//

CLASS OdbcStmt

   DATA   hStmt                   AS NUMERIC PROTECTED
   DATA   oOdbc
   DATA   aFields
   DATA   nRetCode
   DATA   cLastComm   

   METHOD New( oOdbc ) CONSTRUCTOR

   METHOD Execute( cCommand )
   
   METHOD Prepare( cCommand )
   
   METHOD ParamData( pToken )

	METHOD odbcFName( nField )

   METHOD odbcField( nField )

   METHOD odbcFType( nField )

   METHOD odbcFLen( nField )

   METHOD odbcFDec( nField )

   MESSAGE FCount METHOD _FCount()

   METHOD End()

   METHOD odbcFGet( nField )
   
   METHOD SetParam( nField, nTypeC, nType, nLen, nScale, uValue, nValue )   

   METHOD odbcFGetIm( nField, cFich ) 
   
   METHOD PutData( pData, nLen )

   METHOD InitFields()

   METHOD LastError()

   METHOD Skip()
   
   METHOD LastComman() INLINE ( ::cLastComm )
   
   METHOD SetOptions( nType, uBuffer)
   
   METHOD GetOptions( nType)
   
   METHOD SetCurName( cName )

   METHOD GetCurName( )
   
   METHOD RowCount( )
   
   METHOD Cancel()

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( oOrigen ) CLASS OdbcStmt

   local nRet
   local hStmt

   ::nRetCode := 0

   if oOrigen==nil
      ::nRetCode = 1000
      return nil
   else
      ::oOdbc = oOrigen
   endif

   if ( nRet := SQLAllocStmt( ::oOdbc:hDbc, @hStmt ) ) == SQL_SUCCESS
      ::hStmt = hStmt
   else
      ::nRetCode = nRet
      MsgStop( "SQLALLOCSTMT Error" )
      return nil
   endif

return 0

//----------------------------------------------------------------------------//

METHOD End() CLASS OdbcStmt

   SQLFreeStmt( ::hStmt, SQL_DROP )

   ::hStmt = 0

return nil

//----------------------------------------------------------------------------//

METHOD Execute(cCommand) CLASS OdbcStmt

   local nRet := 0

   if cCommand == nil
      ::nRetCode := SQLExecute( ::hStmt )
      nRet:=::nRetCode
      
   else
   
      if Len( cCommand ) < 1
      
         nRet = 2
         
      else
      
         ::cLastComm = cCommand
         if ( nRet := SQLExecDirect( ::hStmt, cCommand ) ) != SQL_SUCCESS
         
            ::nRetCode = nRet
            MsgInfo( "SQLEXECDIRECT Error" + CRLF + ::LastError() )
            
         endif
      endif
      
   endif

return nRet

//----------------------------------------------------------------------------//

METHOD ParamData( pToken ) CLASS OdbcStmt

   ::nRetCode:=SQLParamData( ::hStmt, pToken )

RETURN ::nRetCode

//----------------------------------------------------------------------------//

METHOD Prepare(cCommand) CLASS OdbcStmt


   ::cLastComm = cCommand
   
   ::nRetCode := SQLPrepare( ::hStmt, cCommand )

return ::nRetCode

//----------------------------------------------------------------------------//

METHOD odbcFName ( nField ) CLASS OdbcStmt

return ::aFields[ nField ][ SQLNAME ]

//----------------------------------------------------------------------------//

METHOD odbcField ( nField ) CLASS OdbcStmt

return ::aFields[ nField ][ SQLNAME ]

//----------------------------------------------------------------------------//

METHOD odbcFType( nField ) CLASS OdbcStmt

return ::aFields[ nField ][ SQLCTYPE ]

//----------------------------------------------------------------------------//

METHOD odbcFLen ( nField ) CLASS OdbcStmt

return ::aFields[ nField ][ SQLLEN ]

//----------------------------------------------------------------------------//

METHOD odbcFDec ( nField ) CLASS OdbcStmt

return ::aFields[ nField ][ SQLDEC ]

//----------------------------------------------------------------------------//

METHOD _FCount() CLASS OdbcStmt

   local nFields

   if SQLNumResultCols( ::hStmt, @nFields ) != SQL_SUCCESS
      MsgStop( "SQLNUMRESULTCOLS Error in OdbcStmt:FCount()" )
      return 0
   endif

return nFields

//----------------------------------------------------------------------------//

METHOD _FieldName( nField ) CLASS OdbcStmt

   local cName, nType, nLen, nDec

   SQLDescribeCol( ::hStmt, nField, @cName, @nType, @nLen, @nDec )

return cName

//----------------------------------------------------------------------------//

METHOD InitFields() CLASS OdbcStmt

   local n, nFields := ::FCount()
   local cName, nType, nLen, nDec, lNull

   ::aFields = {}

   for n = 1 to nFields
   
      if ( SQLDescribeCol( ::hStmt, n, @cName, @nType, @nLen, @nDec, @lNull ) != SQL_SUCCESS )
        MsgInfo( "ERROR: " + cName + ::LastError() )
      else
         if ( nType == SQL_DOUBLE .or. nType == SQL_FLOAT ) .and. nDec == 0
            nDec = Set( _SET_DECIMALS )
         endif
         AAdd( ::aFields, { cName, cSQLType( nType ), nLen, nDec, lNull, nType } )
      endif
      
      
   next

return nil

//----------------------------------------------------------------------------//

METHOD odbcFGet( nField ) CLASS OdbcStmt

   local uData
   local nType     := ::aFields[ nField ][ SQLNTYPE ]
   local nSpecType := 0
   local nLen      := ::aFields[ nField ][ SQLLEN ]
   local cHelp1
   local cHelp2
   
   //a nLen se le debe aadir 1 si es nType
   nLen:= If(nLen > 65000, 64000, nLen + 1 )
   
   nSpecType = nType
   
   /* conversiones de tipos especiales */

   do case
      case nType == SQL_VARCHAR .or. nType == SQL_LONGVARCHAR                        
           nSpecType := 0

      case nType == SQL_FLOAT
           nSpecType := 0
           
      case nType == SQL_LONGVARBINARY   
           nSpecType :=0
         
   endcase
            
   if (::nRetCode := SQLGetData( ::hStmt, nField, nSpecType, nLen, @uData ) ) != SQL_SUCCESS
      MsgAlert( "[Imposible recuperar valor de campo "+::aFields[nField][SQLNAME]+;
                "]" + CRLF + ::LastError() )
   endif
   
   do case
      case nType == SQL_CHAR        .or. nType == SQL_LONGVARCHAR .or. ;
           nType == SQL_VARCHAR      
           //no necesita conversion
         
      case nType == SQL_BIT
         uData = If( Asc( SubStr( uData, 1, 1 ) ) == 1, .t., .f. )
         
      case nType == SQL_NUMERIC .or. nType == SQL_DECIMAL
         uData = Val( uData )
       
      case nType == SQL_SMALLINT
         uData = Bin2I( uData )
           
      case nType == SQL_INTEGER
         uData = Bin2L( uData )
           
      case nType == SQL_DOUBLE .or. nType == SQL_FLOAT .or. ;
           nType == SQL_REAL
         uData = Bin2D( uData )
         
      case nType == SQL_DATE
         if Len( uData ) == 0
            uData = CToD( "0" )
          else  
            if Len( uData) == 6
               cHelp1 = Set( _SET_DATEFORMAT )
               SET DATE TO JAPANESE  // "yyyymmdd"
               uData = Str( Bin2I( SubStr( uData, 1, 2 ) ), 4, 0 ) + "/" + ;
                       Str( Bin2I( SubStr( uData, 3, 2 ) ), 2, 0 ) + "/" + ;
                       Str( Bin2I( SubStr( uData, 5, 2 ) ), 2, 0 )
               uData = StrTran( uData, " ", "0" )
               uData = CToD( uData )
               Set( _SET_DATEFORMAT, cHelp1 )
            endif   
         endif   
         
      case nType == SQL_TIME     
         uData = Str( Bin2I( SubStr( uData, 1, 2 ) ), 2, 0 ) + ":" + ;
                 Str( Bin2I( SubStr( uData, 3, 2 ) ), 2, 0 ) + ":" + ;
                 Str( Bin2I( SubStr( uData, 5, 2 ) ), 2, 0 )
         uData = StrTran( uData, " ", "0" )
         
      case nType == SQL_TIMESTAMP
         //el campo se ha creado en la base como C-23     
         //EL DRIVER DEVUELVE DE LONGITUD REAL 16 CARACTERES DE LOS CUALES
         //PARA LA FECHA SOLO TOMAMOS EN CONSIDERACION LOS 6 PRIMEROS 
         //YA QUE EL RESTO CORREPONDE A UNA ESTRUCTURA TIME
         uData = Str( Bin2I( SubStr( uData,  1, 2 ) ), 4, 0 ) + "/" + ;
                 Str( Bin2I( SubStr( uData,  3, 2 ) ), 2, 0 ) + "/" + ;
                 Str( Bin2I( SubStr( uData,  5, 2 ) ), 2, 0 ) + "-" + ;
                 Str( Bin2I( SubStr( uData,  7, 2 ) ), 2, 0 ) + ":" + ;
                 Str( Bin2I( SubStr( uData,  9, 2 ) ), 2, 0 ) + ":" + ;
                 Str( Bin2I( SubStr( uData, 11, 2 ) ), 2, 0 )
         uData = StrTran( uData, " ", "0" )
        
   endcase

return uData

//----------------------------------------------------------------------------//

METHOD odbcFGetIm( nField, cFile ) CLASS OdbcSt

   local uData := ""
   local cRet:=space(0)
   local nType:=0  //field of type binary
   //local nLen:=::aFields[nField][SQLLEN]
   local nLen
   local nh := 0
   local lexit:=.f.
   
   //nLen:= If(nLen > 65000, 64000, nLen + 1 )
   nLen:= 4000
   
   if cFile != NIL  
      nh := fcreate( cFile, 0 )
   endif
   
   do while !lexit
      
      uData := ""
      ::nRetCode:=SQLGetData( ::hStmt, nField, nType, nLen, @uData)         
      
      do case
         case ::nRetCode == SQL_SUCCESS
            //MSGINFO("SUCCESS "+STR(LEN(UDATA)))
            lexit:=.t.
            if cFile!=NIL  
               fwrite( nh, udata, len(udata) )
             else  
               cRet:=cRet+uData
            endif
      
         case ::nRetCode == SQL_SUCCESS_WITH_INFO
            //MSGINFO("SUCCESS_WITH_INFO "+STR(LEN(UDATA)))
            if cFile!=NIL                                             
               fwrite( nh, udata, len(udata) )
             else  
               cRet:=cRet+uData
            endif
            
         otherwise
            lexit:=.t.                
            //MSGINFO("ERROR"+STR(LEN(UDATA))+"_"+CRLF+::LASTERROR())

            
      endcase
            
   enddo
   
   if cFile!=NIL  
      fclose( nh )
   endif

return uData
//----------------------------------------------------------------------------//

function cSQLType( nType )

   local cType := "U"

   do case
      case nType == SQL_CHAR     .OR. nType == SQL_VARCHAR
           cType := "C"

      case nType == SQL_BIT
           cType := "L"

      case nType == SQL_NUMERIC  .OR. nType == SQL_DECIMAL  .OR. ;
           nType == SQL_INTEGER  .OR. nType == SQL_SMALLINT .OR. ; 
           nType == SQL_FLOAT    .OR. nType == SQL_REAL     .OR. ;
           nType == SQL_DOUBLE
           cType := "N"

      case nType == SQL_DATE 
           cType := "D"
           
      case nType == SQL_TIME     .or. nType == SQL_TIMESTAMP     
           cType := "C"

      case nType == SQL_LONGVARCHAR .or. nType == SQL_LONGVARBINARY
           cType := "M"
           
   endcase
   
return cType

//----------------------------------------------------------------------------//

METHOD LastError() CLASS OdbcStmt

   local cClassError, nType, cMsgError

   SQLError( ::oODbc:hEnv, ::oODbc:hDbc, ::hStmt, @cClassError, @nType, @cMsgError )

return cClasserror+" - "+alltrim(str(nType))+" - "+cMsgError

//----------------------------------------------------------------------------//

METHOD skip() CLASS OdbcStmt

  local nRet

  if ( nRet:= SQLFetch( ::hStmt ) ) != SQL_SUCCESS
     //   MsgAlert( "Impossible to Skip" + CRLF + ::LastError() )
  endif

return nRet

//----------------------------------------------------------------------------//

METHOD GetOptions ( nType ) CLASS OdbcStmt

   local cBuffer := Space( 256 )
   
   ::nRetCode := SQLGetStmtO( ::hStmt, nType, @cBuffer )

return cBuffer

//----------------------------------------------------------------------------//

METHOD SetOptions ( nType, uBuffer ) CLASS OdbcStmt
  
return ( ::nRetCode := SQLSetStmtO( ::hStmt, nType, uBuffer ) )

//----------------------------------------------------------------------------//

METHOD SetCurName ( cName ) CLASS OdbcStmt
  
return ( ::nRetCode := SQLSetCurn( ::hStmt, cName ) )

//----------------------------------------------------------------------------//

METHOD GetCurName() CLASS OdbcStmt

   local cBuffer := Space( 256 )
   
   ::nRetCode := SQLGetCurN( ::hStmt, @cBuffer )

return cBuffer

//----------------------------------------------------------------------------//

METHOD Cancel() CLASS OdbcStmt

return ( ::nRetCode := SQLCancel( ::hStmt ) )

//----------------------------------------------------------------------------//
                                         
METHOD RowCount() CLASS OdbcStmt

   local nRow := 0
   
   ::nRetCode := SQLRowCount( ::hStmt, @nRow )

return nRow

//----------------------------------------------------------------------------//
                                         
METHOD SetParam( nField, nTypeC, nType, nLen, nScale, uValue, nValue ) CLASS OdbcStmt

   local uData

   ::nRetCode := SQLSetParam( ::hStmt, nField, nTypeC, nType, nLen,;
                              nScale, uValue, nValue )

return ::nRetCode

//----------------------------------------------------------------------------//

METHOD PutData( pData, nLen ) CLASS OdbcStmt

::nRetCode := SQLPutData( ::hStmt, pData, nLen)

return ::nRetCode
