#include "FiveWin.ch"
#include "Constant.ch"

#define COLOR_WINDOW         5
#define COLOR_WINDOWTEXT     8

#define FD_BORDER            8
#define FD_HEIGHT           22

#define DT_CENTER            1
#define DT_VCENTER           4

#define WINDING              2
#define SC_KEYMENU       61696   // 0xF100

//----------------------------------------------------------------------------//

CLASS TTabs FROM TControl

   CLASSDATA lRegistered AS LOGICAL

   DATA   aPrompts, aSizes
   DATA   nOption
   DATA   bAction

   METHOD New( nTop, nLeft, aPrompts, bAction, oWnd, nOption, nClrFore,;
               nClrBack, lPixel, lDesign, nWidth, nHeight,;
               cMsg ) CONSTRUCTOR

   METHOD ReDefine( nId, aPrompts, bAction, oWnd, nOption, nClrFore,;
                    nClrBack ) CONSTRUCTOR

   METHOD Display()
   METHOD Paint()
   METHOD Init( hDlg )
   METHOD LButtonDown( nRow, nCol, nFlags )
   METHOD Default()
   METHOD AddItem( cItem )

   METHOD DelItem()

   METHOD SetOption( nOption )

   METHOD GetHotPos( nChar )

   METHOD SysCommand( nType, nLoWord, nHiWord )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( nTop, nLeft, aPrompts, bAction, oWnd, nOption, nClrFore,;
            nClrBack, lPixel, lDesign, nWidth, nHeight, cMsg ) CLASS TTabs

   DEFAULT aPrompts := { "&One", "&Two", "T&hree" },;
           nOption  := 1,;
           nClrFore := oWnd:nClrText,;
           nClrBack := CLR_LIGHTGRAY,;
           lPixel   := .f.,;
           lDesign  := .f.,;
           nWidth   := 200, nHeight := 21

   ::nStyle    = nOR( WS_CHILD, WS_VISIBLE,;
                      If( lDesign, WS_CLIPSIBLINGS, 0 ), WS_TABSTOP )
   ::nId       = ::GetNewId()
   ::oWnd      = oWnd
   ::aPrompts  = aPrompts
   ::bAction   = bAction
   ::nOption   = nOption
   ::cMsg      = cMsg
   ::nTop      = If( lPixel, nTop, nTop * SAY_CHARPIX_H )
   ::nLeft     = If( lPixel, nLeft, nLeft * SAY_CHARPIX_W )
   ::nBottom   = ::nTop + nHeight - 1
   ::nRight    = ::nLeft + nWidth - 1
   ::lDrag     = lDesign
   ::lCaptured = .f.
   ::oFont     = TFont():New( "Ms Sans Serif", 0, -9 )

   DEFINE BRUSH ::oBrush STYLE TABS

   // ::SetColor( nClrFore, nClrBack )

   ::Register()

   if ! Empty( oWnd:hWnd )
      ::Create()
      oWnd:AddControl( Self )
   else
      oWnd:DefControl( Self )
   endif

   ::Default()

return nil

//----------------------------------------------------------------------------//

METHOD ReDefine( nId, aPrompts, bAction, oWnd, nOption, nClrFore,;
                 nClrBack ) CLASS TTabs

   local n, oDlg

   DEFAULT nOption  := 1,;
           nClrFore := oWnd:nClrText,;
           nClrBack := oWnd:nClrPane

   ::nId      = nId
   ::oWnd     = oWnd
   ::aPrompts = aPrompts
   ::bAction  = bAction
   ::nOption  = nOption
   ::oFont    = TFont():New( "Ms Sans Serif", 0, -9 )

   ::Register()

   // ::SetColor( nClrFore, nClrBack )

   oWnd:DefControl( Self )

return nil

//----------------------------------------------------------------------------//

METHOD Display() CLASS TTabs

   ::BeginPaint()
   ::Paint()
   ::EndPaint()

return 0

//----------------------------------------------------------------------------//

METHOD Paint() CLASS TTabs

   // TabsPaint( ::hWnd, ::hDC, ::aPrompts, ::aSizes, oFont:hFont,;
   //            ::oFont2:hFont, ::nClrPane, ::nOption )

   local hDarkPen := CreatePen( PS_SOLID, 1, RGB( 128, 128, 128 ) )
   local hGrayPen := CreatePen( PS_SOLID, 1, RGB( 192, 192, 192 ) )
   local hOldPen
   local n, nCol := 5
   local hOldFont, hOldBrush
   local hDC := ::GetDC()

   hOldFont  = SelectObject( hDC, ::oFont:hFont )
   hOldPen   = SelectObject( hDC, hDarkPen )
   hOldBrush = SelectObject( hDC, GetStockObject( If( n == ::nOption, 1, 2 ) ) )

   MoveTo( hDC, 0, 0 )
   LineTo( hDC, ::nRight, 0 )

   SetBlackPen( hDC )
   MoveTo( hDC, 0, 1 )
   LineTo( hDC, ::nRight, 1 )

   SetTextColor( hDC, CLR_WHITE )

   for n = 1 to Len( ::aPrompts )
      SelectObject( hDC, GetStockObject( If( n == ::nOption, 1, 2 ) ) )
      SetBlackPen( hDC )
      if n == 1 .or. n == ::nOption
         PolyPolygon( hDC, { { nCol, 1 },;
                               { nCol + 5, 15 },;
                               { nCol + 9 + ::aSizes[ n ], 15 },;
                               { nCol + 14 + ::aSizes[ n ], 1 },;
                               { nCol, 1 } } )
      else
         PolyPolygon( hDC, { { nCol + 2, 7 },;
                               { nCol + 5, 15 },;
                               { nCol + 9 + ::aSizes[ n ], 15 },;
                               { nCol + 14 + ::aSizes[ n ], 1 },;
                               { nCol + 4, 1 },;
                               { nCol + 2, 7 } } )
      endif

      if n == ::nOption
         SelectObject( hDC, hGrayPen )
         MoveTo( hDC, nCol, 0 )
         LineTo( hDC, nCol + 14 + ::aSizes[ n ], 0 )
         MoveTo( hDC, nCol + 1, 1 )
         LineTo( hDC, nCol + 14 + ::aSizes[ n ], 1 )

         SetWhitePen( hDC )
         MoveTo( hDC, nCol + 1,  2 )
         LineTo( hDC, nCol + 6, 14 )

         SelectObject( hDC, hDarkPen )
         LineTo( hDC, nCol + ::aSizes[ n ] +  8, 14 )
         LineTo( hDC, nCol + ::aSizes[ n ] + 13,  0 )

         SetBkColor( hDC, RGB( 192, 192, 192 ) )
         DrawText( hDC, ::aPrompts[ n ],;
                   { 1, nCol + 4, 15, nCol + 5 + ::aSizes[ n ] + 5 },;
                   nOr( DT_CENTER, DT_VCENTER ) )
      else
         SetBkColor( hDC, RGB( 128, 128, 128 ) )
         DrawText( hDC, ::aPrompts[ n ],;
                   { 2, nCol + 3, 19, nCol + 5 + ::aSizes[ n ] + 6 },;
                   nOr( DT_CENTER, DT_VCENTER ) )
      endif
      nCol += ::aSizes[ n ] + 4 + 6
   next

   SelectObject( hDC, hOldPen )
   SelectObject( hDC, hOldFont )
   SelectObject( hDC, hOldBrush )
   DeleteObject( hDarkPen )
   DeleteObject( hGrayPen )

   ::ReleaseDC()

return nil

//----------------------------------------------------------------------------//

METHOD Init( hDlg ) CLASS TTabs

   Super:Init( hDlg )

   DEFINE BRUSH ::oBrush STYLE TABS

   ::Default()

return nil

//----------------------------------------------------------------------------//

METHOD LButtonDown( nRow, nCol, nFlags ) CLASS TTabs

   local n := 1, nPos := 5

   if ::lDrag
      return Super:LButtonDown( nRow, nCol, nFlags )
   else
      if nRow <= FD_HEIGHT
         while nCol > nPos + ::aSizes[ n ] + 10 .and. n < Len( ::aPrompts )
            nPos += ::aSizes[ n ] + 10
            n++
         end
         ::SetOption( n )
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Default() CLASS TTabs

   local n

   ::aSizes = Array( Len( ::aPrompts ) )

   for n = 1 to Len( ::aPrompts )
      ::aSizes[ n ] = GetTextWidth( 0, StrTran( ::aPrompts[ n ], "&", "" ),;
                                    ::oFont:hFont )
   next

return nil

//----------------------------------------------------------------------------//

METHOD AddItem( cItem ) CLASS TTabs

   AAdd( ::aPrompts, cItem )
   ::Default()
   ::Refresh()

return nil

//----------------------------------------------------------------------------//

METHOD DelItem() CLASS TTabs

   if Len( ::aPrompts ) > 1
      ::aPrompts = ADel( ::aPrompts, ::nOption )
      ::aPrompts = ASize( ::aPrompts, Len( ::aPrompts ) - 1 )
   else
      ::aPrompts = { "No Defined" }
   endif
   ::Default()
   ::Refresh()

return nil

//----------------------------------------------------------------------------//

METHOD SetOption( nOption ) CLASS TTabs

   if nOption != ::nOption
      ::nOption = nOption
      ::Paint()
      if ! Empty( ::bAction )
         Eval( ::bAction )
      endif
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GetHotPos( nChar ) CLASS TTabs

   local n := 1
   local nAt

   while n <= Len( ::aPrompts )
     if ( nAt := At( "&", ::aPrompts[ n ] ) ) != 0 .and. ;
        Lower( SubStr( ::aPrompts[ n ], nAt + 1, 1 ) ) == Chr( nChar )
        return n
     endif
     n++
   end

return 0

//----------------------------------------------------------------------------//

METHOD SysCommand( nType, nLoWord, nHiWord ) CLASS TTabs

   do case
      case nType == SC_KEYMENU      // Alt+... control accelerator pressed
           ::SetOption( ::GetHotPos( nLoWord ) )
           return 0
   endcase

return nil

//----------------------------------------------------------------------------//
