#include "FiveWin.ch"
#include "FileIO.ch"

#define DEFAULT_BUFFER_LEN  2000

//----------------------------------------------------------------------------//

CLASS TFile

   DATA   cName, cBuffer
   DATA   nHandle
   DATA   nLength, nBufLen, nBufPos
   DATA   lEoF AS LOGICAL

   METHOD New( cFileName ) CONSTRUCTOR
   METHOD Close()

   METHOD cGetToken()
   METHOD LoadBuffer()
   METHOD Skip( nBytes )

   METHOD GoTop()    INLINE  FSeek( ::hFile, 0, FS_SET ), ::LoadBuffer()
   METHOD GoBottom() INLINE  FSeek( ::hFile, 0, FS_END )

   METHOD EoF()      INLINE  ::lEoF

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( cFileName ) CLASS TFile

   ::cName = cFileName

   if File( cFileName )
      ::nHandle = FOpen( cFileName )
      ::nLength = FSeek( ::nHandle, 0, FS_END )
      FSeek( ::nHandle, 0, FS_SET )
      ::nBufLen = Min( DEFAULT_BUFFER_LEN, ::nLength )
   else
      ::nHandle = FCreate( cFileName )
      ::nLength = 0
      ::nBufLen = DEFAULT_BUFFER_LEN
   endif

   ::LoadBuffer()

return nil

//----------------------------------------------------------------------------//

METHOD Close() CLASS TFile

   FClose( ::nHandle )
   ::nHandle = 0
   ::cBuffer = nil

return nil

//----------------------------------------------------------------------------//

METHOD cGetToken() CLASS TFile

   local nStart, nEnd
   local cChar

   while ( ! ::EoF() ) .and. ;
         ( ( cChar := SubStr( ::cBuffer, ::nBufPos, 1 ) ) == " " .or. ;
             cChar == Chr( 13 ) .or. cChar == Chr( 10 ) )
      ::Skip()
   end
   nStart = ::nBufPos

   while ( ! ::EoF() ) .and. ;
         ( ( cChar := SubStr( ::cBuffer, ::nBufPos, 1 ) ) != " " .and. ;
           cChar != Chr( 13 ) )
      ::Skip()
   end
   nEnd = ::nBufPos - If( cChar == Chr( 13 ), 1, 0 )

return SubStr( ::cBuffer, nStart, nEnd - nStart + 1 )

//----------------------------------------------------------------------------//

METHOD LoadBuffer() CLASS TFile

   local cTemp := Space( ::nBufLen )

   if ( ::nBufLen := FRead( ::nHandle, @cTemp, Len( cTemp ) ) ) == 0
      ::lEoF = .t.
   endif

   ::cBuffer = cTemp
   ::nBufPos = 1

return nil

//----------------------------------------------------------------------------//

METHOD Skip( nBytes ) CLASS TFile

   DEFAULT nBytes := 1

   if ( ::nBufPos += nBytes ) >= ::nBufLen .and. ! ::lEoF
      ::LoadBuffer()
   endif

   if ::nBufPos >= ::nBufLen
      ::lEoF = .t.
   endif

return nil

//----------------------------------------------------------------------------//
