#include "FiveWin.ch"
#include "objects.ch"

#define MM_ANISOTROPIC         8

#define SW_HIDE                0
#define SW_SHOWNA              8

#define SHADOW_DEEP            5
#define SHADOW_WIDTH           5

static lRegistered := .f.

//----------------------------------------------------------------------------//

CLASS TMetaFile FROM TControl

   DATA   hMeta
   DATA   oPen
   DATA   nWidth, nHeight, nXorig, nYorig
   DATA   lZoom, lShadow

   METHOD New( nTop, nLeft, nWidth, nHeight, cMetaFile, oWnd,;
               nClrFore, nClrBack ) CONSTRUCTOR

   METHOD Redefine( nId, cMetaFile, oWnd, nClrFore, nClrBack ) CONSTRUCTOR

   METHOD Display() INLINE ::BeginPaint(), ::Paint(), ::EndPaint(), 0
   METHOD Paint()

   METHOD SetFile(cFile)

   METHOD Shadow(lOn)

   METHOD Hide() INLINE (::Shadow(.F.),ShowWindow( ::hWnd, SW_HIDE ))

   METHOD Setcoors(oCoors) INLINE (::Shadow(.F.)            ,;
                                   Super:Setcoors(oCoors))

   METHOD End()

   METHOD ZoomIn()  INLINE IIF(!::lZoom, (::nWidth  /= 2 ,;
                                          ::nHeight /= 4   ,;
                                          ::lZoom   := .T. ,;
                                          ::Refresh()), )
   METHOD ZoomOut() INLINE IIF(::lZoom , (::nWidth  *= 2 ,;
                                          ::nHeight *= 4   ,;
                                          ::lZoom   := .F. ,;
                                          ::nXorig  := 0   ,;
                                          ::nYorig  := 0   ,;
                                          ::Refresh()), )

   METHOD SetOrg(nX,nY)    INLINE iif(nX != NIL, ::nXorig := nX ,) ,;
                                  iif(nY != NIL, ::nYorig := nY ,)

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( nTop, nLeft, nWidth, nHeight, cMetaFile, oWnd,;
            nClrFore, nClrBack,nLogWidth, nLogHeight ) CLASS TMetaFile

   DEFAULT nWidth := 100, nHeight := 100, oWnd := GetWndDefault()

   ::nTop     = nTop
   ::nLeft    = nLeft
   ::nBottom  = nTop + nHeight - 1
   ::nRight   = nLeft + nWidth - 1
   ::cCaption = cMetaFile
   ::oWnd     = oWnd
   ::nStyle   = nOr( WS_CHILD, WS_BORDER, WS_VISIBLE )
   ::nWidth   = nLogWidth
   ::nHeight  = nLogHeight
   ::lZoom    = .F.
   ::lShadow  = .T.
   ::nXorig   = 0
   ::nYorig   = 0
   ::hMeta    = 0

   if ! lRegistered
      ::Register()
      lRegistered = .t.
   endif

   ::SetColor( nClrFore, nClrBack )

   if ::lShadow
     DEFINE PEN ::oPen WIDTH SHADOW_WIDTH
   endif

   if oWnd:lVisible
      ::Create()
      ::Default()
      ::lVisible = .t.
      oWnd:AddControl( Self )
   else
      oWnd:DefControl( Self )
      ::lVisible  = .f.
   endif

return nil

//----------------------------------------------------------------------------//

METHOD Redefine( nId, cMetaFile, oWnd, nClrFore, nClrBack ) CLASS TMetaFile

   DEFAULT oWnd := GetWndDefault()

   ::nId      = nId
   ::cCaption = cMetaFile
   ::oWnd     = oWnd
   ::nWidth   = 100
   ::nHeight  = 100
   ::hMeta    = 0

   if ! lRegistered
      ::Register()
      lRegistered = .t.
   endif

   ::SetColor( nClrFore, nClrBack )

   oWnd:DefControl( Self )

return nil

//----------------------------------------------------------------------------//

METHOD Paint() CLASS TMetaFile

   local oRect := ::GetRect()

   IF ::hMeta == 0
        IF file(::cCaption)
           ::hMeta := GetMetaFile( ::cCaption )
        ELSEIF !empty(::cCaption)
           Alert("Could not find the Metafile,"+CRLF+;
                 "please check your TEMP environment variable")
        ENDIF
   ENDIF


   IF ::hMeta != 0
        ::Shadow(.T.)
        SetWindowOrg(::hDC, ::nXorig, ::nYorig)
        SetMapMode( ::hDC, MM_ANISOTROPIC )
        SetWindowExt( ::hDC, ::nWidth, ::nHeight )
        SetViewportExt( ::hDC, oRect:nRight - oRect:nLeft,;
                        oRect:nBottom - oRect:nTop )
        PlayMetaFile( ::hDC, ::hMeta )
   ENDIF

return nil

//----------------------------------------------------------------------------//
METHOD SetFile(cFile) CLASS TMetaFile

     IF file(cFile)
          ::cCaption := cFile
     ELSE
          ::cCaption := ""
     ENDIF

     IF ::hMeta != 0
          DeleteMetafile(::hMeta)
          ::hMeta := 0
     ENDIF

RETURN NIL

//----------------------------------------------------------------------------//

METHOD Shadow(lOn) CLASS TMetaFile

     DEFAULT lOn := .T.

     IF !::lShadow
          RETU NIL
     ENDIF

     IF lOn
          ::oWnd:GetDC()
          MoveTo( ::oWnd:hDC              ,;
                  ::nLeft + SHADOW_DEEP   ,;
                  ::nBottom )
          LineTo( ::oWnd:hDC              ,;
                  ::nRight                ,;
                  ::nBottom               ,;
                  ::oPen:hPen )
          MoveTo( ::oWnd:hDC              ,;
                  ::nRight                ,;
                  ::nTop + SHADOW_DEEP )
          LineTo( ::oWnd:hDC              ,;
                  ::nRight                ,;
                  ::nBottom               ,;
                  ::oPen:hPen )
          ::oWnd:ReleaseDC()
     ELSE
          InvalidateRect(::oWnd:hWnd            ,;
                        {::nTop                 ,;
                         ::nLeft                ,;
                         ::nBottom+SHADOW_DEEP  ,;
                         ::nRight+SHADOW_WIDTH}  )
     ENDIF


RETURN NIL

//----------------------------------------------------------------------------//
METHOD End() CLASS TMetaFile

     IF ::hMeta != 0
          DeleteMetafile(::hMeta)
          ::hMeta := 0
     ENDIF

     ::Shadow(.F.)

     IF ::lShadow
          ::oPen:End()
     ENDIF

     Super:End()

RETURN NIL

