#include "FiveWin.ch"

//----------------------------------------------------------------------------//

CLASS TWorkArea

   DATA   nArea          AS NUMERIC             // Don't modify the order !!!
   DATA   lBuffer        AS LOGICAL READONLY    // of this !!!
   DATA   aBuffer        AS ARRAY               // DATA !!!

   METHOD New( nArea )  INLINE ( If( ! Empty( nArea ), nArea, Select() ) )->( ::SetArea() ), Self

   ERROR HANDLER TWAErrHand EXTERN

   METHOD Blank( nRecNo )     INLINE ( ::nArea )->( nRecNo := RecNo(),;
                                                    DBGoBottom(), ;
                                                    DBSkip( 1 ), ;
                                                    ::Load(),;
                                                    DBGoTo( nRecNo ) )
   METHOD Load()              EXTERN TWALoad()
   METHOD Save()              EXTERN TWASave()
   METHOD SetBuffer( lOnOff )      // if TRUE reloads buffer, return lBuffer.

   HIDE METHOD SetArea()      // Open at nArea and self-Init .
   HIDE METHOD ResetArea()    VIRTUAL

   METHOD Append()            INLINE ( ::nArea )->( DbAppend() )

   METHOD Bof()               EXTERN TWABof()
   METHOD Close()             INLINE ( ::nArea )->( DbCloseArea() ), ::ResetArea()
   METHOD Commit()            INLINE ( ::nArea )->( DBCommit() )
   METHOD Create( cFile, aStruct, cDriver ) ;
                              INLINE DbCreate( cFile, aStruct, cDriver )
   METHOD DbEval( bBlock, bFor, bWhile, nNext, nRecord, lRest ) ;
                              INLINE ( ::nArea )->( DBEval( bBlock, bFor, ;
                                                    bWhile, nNext, nRecord, ;
                                                    lRest ) )
   METHOD Delete()            INLINE ( ::nArea )->( DBDelete() )
   METHOD Deleted()           EXTERN TWADeleted()
   METHOD Eof()               EXTERN TWAEof()
   METHOD FCount()            EXTERN TWAFCount()
   METHOD FieldGet( nField )       EXTERN TWAFieldGe()  // if buffering
   METHOD FieldPut( nField, uVal ) EXTERN TWAFieldSe()  // access to buffer.
   METHOD Found()             INLINE ( ::nArea )->( Found() )
   METHOD GoTo( nRecNo )      INLINE ( ::nArea )->( DBGoTo( nRecNo ) ),;
                                     if( ::lBuffer, ::Load(), )
   METHOD GoTop()             INLINE ( ::nArea )->( DBGoTop() ),;
                                     if( ::lBuffer, ::Load(), )
   METHOD GoBottom()          INLINE ( ::nArea )->( DBGoBottom() ),;
                                     if( ::lBuffer, ::Load(), )
   METHOD LastRec( nRec )     EXTERN TWALastRec()
   METHOD Lock()              EXTERN TWALock()   // File locking
   METHOD Pack()              EXTERN TWAPack()
   METHOD ReCall()            INLINE ( ::nArea )->( DBRecall() )

   METHOD RecCount()          EXTERN TWALastRec()
   METHOD RecLock()           EXTERN TWARLock()  // Record locking
   METHOD RecNo()             EXTERN TWARecNo()
   METHOD Seek( uExp, lSoft ) EXTERN TWASeek()  // if buffering reloads buffer
   METHOD Skip( nRecords )    EXTERN TWASkip()  //    "
   METHOD Skipper( nRecords ) EXTERN TWASkipper() // -> nRecsSkipped,  "
   METHOD UnLock()            INLINE ( ::nArea )->( DBUnLock() )
   
   METHOD Use( nArea, cDriver, cFile, cAlias, lShared, lReadOnly ) INLINE ;
          ( nArea )->( DbUseArea( .f., cDriver, cFile, cAlias, lShared, lReadOnly ),;
                       ::SetArea() )
          
   METHOD Used()              INLINE ( ::nArea )->( Used() )
   METHOD Zap()               EXTERN TWAZap() // if buffering reloads buffer

   METHOD AddIndex( cFile, cTag ) INLINE ( ::nArea )->( OrdListAdd( cFile, cTag ) )
   METHOD CloseIndex()            INLINE ( ::nArea )->( OrdListClear() )
   METHOD CreateIndex( cFile, cTag, cKey, lUnique ) INLINE ;
                       ( ::nArea )->( OrdCreate( cFile, cTag, cKey,, lUnique ) )
   METHOD ClearRelation()           INLINE ( ::nArea )->( DbClearRelation() )
   METHOD DeleteIndex( cTag, cFile ) INLINE ( ::nArea )->( OrdDestroy( cTag, cFile ) )
   METHOD IndexKey( ncTag, cFile )  INLINE ( ::nArea )->( OrdKey( ncTag, cFile ) )
   METHOD IndexName( nTag, cFile )  INLINE ( ::nArea )->( OrdName( nTag, cFile ) )
   METHOD IndexBagName( nInd )      INLINE ( ::nArea )->( OrdBagName( nInd ) )
   METHOD IndexOrder( cTag, cFile ) INLINE ( ::nArea )->( OrdNumber( cTag, cFile ) )
   METHOD SetOrder( cnTag, cFile )  INLINE ( ::nArea )->( OrdSetFocus( cnTag, cFile ) )
   METHOD SetRelation( ncArea, cExp ) INLINE ;
                 ( ::nArea )->( DbSetRelation( ncArea, Compile( cExp ), cExp ) )
   
ENDCLASS

//---------------------------------------------------------------------------//

static function Compile( cExp )
return &( "{||" + cExp + "}" )

//----------------------------------------------------------------------------//

METHOD SetArea()

   static nClassH
   local  n, nFields := FCount()
   
   DEFAULT nClassH := TWorkArea():ClassH()

   for n = 1 to nFields
       TWADictAdd( nClassH, "_" + Field( n ), Field( n ) )
   next
   ::aBuffer = Array( nFields )
   ::nArea   = Select()
   ::SetBuffer( .t. )
   
return nFields

//---------------------------------------------------------------------------//

METHOD SetBuffer( lOnOff )

   if lOnOff != nil
       ::lBuffer = lOnOff
   endif   

   if ::lBuffer
      ::Load()
   endif   

return ::lBuffer

//----------------------------------------------------------------------------//
